import sys
from datetime import datetime
from typing import List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

AccessPointArn = str
AccessPointId = str
AvailabilityZoneId = str
AvailabilityZoneName = str
AwsAccountId = str
Backup = bool
BypassPolicyLockoutSafetyCheck = bool
ClientToken = str
CreationToken = str
Encrypted = bool
ErrorCode = str
ErrorMessage = str
FileSystemArn = str
FileSystemId = str
IpAddress = str
KmsKeyId = str
Marker = str
MaxItems = int
MaxResults = int
MountTargetCount = int
MountTargetId = str
Name = str
NetworkInterfaceId = str
Path = str
Permissions = str
Policy = str
ProvisionedThroughputInMibps = float
RegionName = str
ResourceId = str
SecurityGroup = str
SubnetId = str
TagKey = str
TagValue = str
Token = str
VpcId = str


class LifeCycleState(str):
    creating = "creating"
    available = "available"
    updating = "updating"
    deleting = "deleting"
    deleted = "deleted"
    error = "error"


class PerformanceMode(str):
    generalPurpose = "generalPurpose"
    maxIO = "maxIO"


class ReplicationStatus(str):
    ENABLED = "ENABLED"
    ENABLING = "ENABLING"
    DELETING = "DELETING"
    ERROR = "ERROR"


class Resource(str):
    FILE_SYSTEM = "FILE_SYSTEM"
    MOUNT_TARGET = "MOUNT_TARGET"


class ResourceIdType(str):
    LONG_ID = "LONG_ID"
    SHORT_ID = "SHORT_ID"


class Status(str):
    ENABLED = "ENABLED"
    ENABLING = "ENABLING"
    DISABLED = "DISABLED"
    DISABLING = "DISABLING"


class ThroughputMode(str):
    bursting = "bursting"
    provisioned = "provisioned"
    elastic = "elastic"


class TransitionToIARules(str):
    AFTER_7_DAYS = "AFTER_7_DAYS"
    AFTER_14_DAYS = "AFTER_14_DAYS"
    AFTER_30_DAYS = "AFTER_30_DAYS"
    AFTER_60_DAYS = "AFTER_60_DAYS"
    AFTER_90_DAYS = "AFTER_90_DAYS"
    AFTER_1_DAY = "AFTER_1_DAY"


class TransitionToPrimaryStorageClassRules(str):
    AFTER_1_ACCESS = "AFTER_1_ACCESS"


class AccessPointAlreadyExists(ServiceException):
    """Returned if the access point that you are trying to create already
    exists, with the creation token you provided in the request.
    """

    code: str = "AccessPointAlreadyExists"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode
    AccessPointId: AccessPointId


class AccessPointLimitExceeded(ServiceException):
    """Returned if the Amazon Web Services account has already created the
    maximum number of access points allowed per file system. For more
    informaton, see
    https://docs.aws.amazon.com/efs/latest/ug/limits.html#limits-efs-resources-per-account-per-region.
    """

    code: str = "AccessPointLimitExceeded"
    sender_fault: bool = False
    status_code: int = 403
    ErrorCode: ErrorCode


class AccessPointNotFound(ServiceException):
    """Returned if the specified ``AccessPointId`` value doesn't exist in the
    requester's Amazon Web Services account.
    """

    code: str = "AccessPointNotFound"
    sender_fault: bool = False
    status_code: int = 404
    ErrorCode: ErrorCode


class AvailabilityZonesMismatch(ServiceException):
    """Returned if the Availability Zone that was specified for a mount target
    is different from the Availability Zone that was specified for One Zone
    storage. For more information, see `Regional and One Zone storage
    redundancy <https://docs.aws.amazon.com/efs/latest/ug/availability-durability.html>`__.
    """

    code: str = "AvailabilityZonesMismatch"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: Optional[ErrorCode]


class BadRequest(ServiceException):
    """Returned if the request is malformed or contains an error such as an
    invalid parameter value or a missing required parameter.
    """

    code: str = "BadRequest"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class DependencyTimeout(ServiceException):
    """The service timed out trying to fulfill the request, and the client
    should try the call again.
    """

    code: str = "DependencyTimeout"
    sender_fault: bool = False
    status_code: int = 504
    ErrorCode: ErrorCode


class FileSystemAlreadyExists(ServiceException):
    """Returned if the file system you are trying to create already exists,
    with the creation token you provided.
    """

    code: str = "FileSystemAlreadyExists"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode
    FileSystemId: FileSystemId


class FileSystemInUse(ServiceException):
    """Returned if a file system has mount targets."""

    code: str = "FileSystemInUse"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class FileSystemLimitExceeded(ServiceException):
    """Returned if the Amazon Web Services account has already created the
    maximum number of file systems allowed per account.
    """

    code: str = "FileSystemLimitExceeded"
    sender_fault: bool = False
    status_code: int = 403
    ErrorCode: ErrorCode


class FileSystemNotFound(ServiceException):
    """Returned if the specified ``FileSystemId`` value doesn't exist in the
    requester's Amazon Web Services account.
    """

    code: str = "FileSystemNotFound"
    sender_fault: bool = False
    status_code: int = 404
    ErrorCode: ErrorCode


class IncorrectFileSystemLifeCycleState(ServiceException):
    """Returned if the file system's lifecycle state is not "available"."""

    code: str = "IncorrectFileSystemLifeCycleState"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class IncorrectMountTargetState(ServiceException):
    """Returned if the mount target is not in the correct state for the
    operation.
    """

    code: str = "IncorrectMountTargetState"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class InsufficientThroughputCapacity(ServiceException):
    """Returned if there's not enough capacity to provision additional
    throughput. This value might be returned when you try to create a file
    system in provisioned throughput mode, when you attempt to increase the
    provisioned throughput of an existing file system, or when you attempt
    to change an existing file system from Bursting Throughput to
    Provisioned Throughput mode. Try again later.
    """

    code: str = "InsufficientThroughputCapacity"
    sender_fault: bool = False
    status_code: int = 503
    ErrorCode: ErrorCode


class InternalServerError(ServiceException):
    """Returned if an error occurred on the server side."""

    code: str = "InternalServerError"
    sender_fault: bool = False
    status_code: int = 500
    ErrorCode: ErrorCode


class InvalidPolicyException(ServiceException):
    """Returned if the ``FileSystemPolicy`` is malformed or contains an error
    such as a parameter value that is not valid or a missing required
    parameter. Returned in the case of a policy lockout safety check error.
    """

    code: str = "InvalidPolicyException"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: Optional[ErrorCode]


class IpAddressInUse(ServiceException):
    """Returned if the request specified an ``IpAddress`` that is already in
    use in the subnet.
    """

    code: str = "IpAddressInUse"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class MountTargetConflict(ServiceException):
    """Returned if the mount target would violate one of the specified
    restrictions based on the file system's existing mount targets.
    """

    code: str = "MountTargetConflict"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class MountTargetNotFound(ServiceException):
    """Returned if there is no mount target with the specified ID found in the
    caller's Amazon Web Services account.
    """

    code: str = "MountTargetNotFound"
    sender_fault: bool = False
    status_code: int = 404
    ErrorCode: ErrorCode


class NetworkInterfaceLimitExceeded(ServiceException):
    """The calling account has reached the limit for elastic network interfaces
    for the specific Amazon Web Services Region. Either delete some network
    interfaces or request that the account quota be raised. For more
    information, see `Amazon VPC
    Quotas <https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_Appendix_Limits.html>`__
    in the *Amazon VPC User Guide* (see the **Network interfaces per
    Region** entry in the **Network interfaces** table).
    """

    code: str = "NetworkInterfaceLimitExceeded"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class NoFreeAddressesInSubnet(ServiceException):
    """Returned if ``IpAddress`` was not specified in the request and there are
    no free IP addresses in the subnet.
    """

    code: str = "NoFreeAddressesInSubnet"
    sender_fault: bool = False
    status_code: int = 409
    ErrorCode: ErrorCode


class PolicyNotFound(ServiceException):
    """Returned if the default file system policy is in effect for the EFS file
    system specified.
    """

    code: str = "PolicyNotFound"
    sender_fault: bool = False
    status_code: int = 404
    ErrorCode: Optional[ErrorCode]


class ReplicationNotFound(ServiceException):
    """Returned if the specified file system does not have a replication
    configuration.
    """

    code: str = "ReplicationNotFound"
    sender_fault: bool = False
    status_code: int = 404
    ErrorCode: Optional[ErrorCode]


class SecurityGroupLimitExceeded(ServiceException):
    """Returned if the size of ``SecurityGroups`` specified in the request is
    greater than five.
    """

    code: str = "SecurityGroupLimitExceeded"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class SecurityGroupNotFound(ServiceException):
    """Returned if one of the specified security groups doesn't exist in the
    subnet's virtual private cloud (VPC).
    """

    code: str = "SecurityGroupNotFound"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class SubnetNotFound(ServiceException):
    """Returned if there is no subnet with ID ``SubnetId`` provided in the
    request.
    """

    code: str = "SubnetNotFound"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class ThrottlingException(ServiceException):
    """Returned when the ``CreateAccessPoint`` API action is called too quickly
    and the number of Access Points on the file system is nearing the `limit
    of
    120 <https://docs.aws.amazon.com/efs/latest/ug/limits.html#limits-efs-resources-per-account-per-region>`__.
    """

    code: str = "ThrottlingException"
    sender_fault: bool = False
    status_code: int = 429
    ErrorCode: Optional[ErrorCode]


class ThroughputLimitExceeded(ServiceException):
    """Returned if the throughput mode or amount of provisioned throughput
    can't be changed because the throughput limit of 1024 MiB/s has been
    reached.
    """

    code: str = "ThroughputLimitExceeded"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class TooManyRequests(ServiceException):
    """Returned if you don’t wait at least 24 hours before either changing the
    throughput mode, or decreasing the Provisioned Throughput value.
    """

    code: str = "TooManyRequests"
    sender_fault: bool = False
    status_code: int = 429
    ErrorCode: ErrorCode


class UnsupportedAvailabilityZone(ServiceException):
    """Returned if the requested Amazon EFS functionality is not available in
    the specified Availability Zone.
    """

    code: str = "UnsupportedAvailabilityZone"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


class ValidationException(ServiceException):
    """Returned if the Backup service is not available in the Amazon Web
    Services Region in which the request was made.
    """

    code: str = "ValidationException"
    sender_fault: bool = False
    status_code: int = 400
    ErrorCode: ErrorCode


OwnerGid = int
OwnerUid = int


class CreationInfo(TypedDict, total=False):
    """Required if the ``RootDirectory`` > ``Path`` specified does not exist.
    Specifies the POSIX IDs and permissions to apply to the access point's
    ``RootDirectory`` > ``Path``. If the access point root directory does
    not exist, EFS creates it with these settings when a client connects to
    the access point. When specifying ``CreationInfo``, you must include
    values for all properties.

    Amazon EFS creates a root directory only if you have provided the
    CreationInfo: OwnUid, OwnGID, and permissions for the directory. If you
    do not provide this information, Amazon EFS does not create the root
    directory. If the root directory does not exist, attempts to mount using
    the access point will fail.

    If you do not provide ``CreationInfo`` and the specified
    ``RootDirectory`` does not exist, attempts to mount the file system
    using the access point will fail.
    """

    OwnerUid: OwnerUid
    OwnerGid: OwnerGid
    Permissions: Permissions


class RootDirectory(TypedDict, total=False):
    """Specifies the directory on the Amazon EFS file system that the access
    point provides access to. The access point exposes the specified file
    system path as the root directory of your file system to applications
    using the access point. NFS clients using the access point can only
    access data in the access point's ``RootDirectory`` and it's
    subdirectories.
    """

    Path: Optional[Path]
    CreationInfo: Optional[CreationInfo]


Gid = int
SecondaryGids = List[Gid]
Uid = int


class PosixUser(TypedDict, total=False):
    """The full POSIX identity, including the user ID, group ID, and any
    secondary group IDs, on the access point that is used for all file
    system operations performed by NFS clients using the access point.
    """

    Uid: Uid
    Gid: Gid
    SecondaryGids: Optional[SecondaryGids]


class Tag(TypedDict, total=False):
    """A tag is a key-value pair. Allowed characters are letters, white space,
    and numbers that can be represented in UTF-8, and the following
    characters:``+ - = . _ : /``.
    """

    Key: TagKey
    Value: TagValue


Tags = List[Tag]


class AccessPointDescription(TypedDict, total=False):
    """Provides a description of an EFS file system access point."""

    ClientToken: Optional[ClientToken]
    Name: Optional[Name]
    Tags: Optional[Tags]
    AccessPointId: Optional[AccessPointId]
    AccessPointArn: Optional[AccessPointArn]
    FileSystemId: Optional[FileSystemId]
    PosixUser: Optional[PosixUser]
    RootDirectory: Optional[RootDirectory]
    OwnerId: Optional[AwsAccountId]
    LifeCycleState: Optional[LifeCycleState]


AccessPointDescriptions = List[AccessPointDescription]


class BackupPolicy(TypedDict, total=False):
    """The backup policy for the file system used to create automatic daily
    backups. If status has a value of ``ENABLED``, the file system is being
    automatically backed up. For more information, see `Automatic
    backups <https://docs.aws.amazon.com/efs/latest/ug/awsbackup.html#automatic-backups>`__.
    """

    Status: Status


class BackupPolicyDescription(TypedDict, total=False):
    BackupPolicy: Optional[BackupPolicy]


class CreateAccessPointRequest(ServiceRequest):
    ClientToken: ClientToken
    Tags: Optional[Tags]
    FileSystemId: FileSystemId
    PosixUser: Optional[PosixUser]
    RootDirectory: Optional[RootDirectory]


class CreateFileSystemRequest(ServiceRequest):
    CreationToken: CreationToken
    PerformanceMode: Optional[PerformanceMode]
    Encrypted: Optional[Encrypted]
    KmsKeyId: Optional[KmsKeyId]
    ThroughputMode: Optional[ThroughputMode]
    ProvisionedThroughputInMibps: Optional[ProvisionedThroughputInMibps]
    AvailabilityZoneName: Optional[AvailabilityZoneName]
    Backup: Optional[Backup]
    Tags: Optional[Tags]


SecurityGroups = List[SecurityGroup]


class CreateMountTargetRequest(ServiceRequest):
    FileSystemId: FileSystemId
    SubnetId: SubnetId
    IpAddress: Optional[IpAddress]
    SecurityGroups: Optional[SecurityGroups]


class DestinationToCreate(TypedDict, total=False):
    """Describes the destination file system to create in the replication
    configuration.
    """

    Region: Optional[RegionName]
    AvailabilityZoneName: Optional[AvailabilityZoneName]
    KmsKeyId: Optional[KmsKeyId]


DestinationsToCreate = List[DestinationToCreate]


class CreateReplicationConfigurationRequest(ServiceRequest):
    SourceFileSystemId: FileSystemId
    Destinations: DestinationsToCreate


class CreateTagsRequest(ServiceRequest):
    FileSystemId: FileSystemId
    Tags: Tags


class DeleteAccessPointRequest(ServiceRequest):
    AccessPointId: AccessPointId


class DeleteFileSystemPolicyRequest(ServiceRequest):
    FileSystemId: FileSystemId


class DeleteFileSystemRequest(ServiceRequest):
    FileSystemId: FileSystemId


class DeleteMountTargetRequest(ServiceRequest):
    MountTargetId: MountTargetId


class DeleteReplicationConfigurationRequest(ServiceRequest):
    SourceFileSystemId: FileSystemId


TagKeys = List[TagKey]


class DeleteTagsRequest(ServiceRequest):
    FileSystemId: FileSystemId
    TagKeys: TagKeys


class DescribeAccessPointsRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[Token]
    AccessPointId: Optional[AccessPointId]
    FileSystemId: Optional[FileSystemId]


class DescribeAccessPointsResponse(TypedDict, total=False):
    AccessPoints: Optional[AccessPointDescriptions]
    NextToken: Optional[Token]


class DescribeAccountPreferencesRequest(ServiceRequest):
    NextToken: Optional[Token]
    MaxResults: Optional[MaxResults]


Resources = List[Resource]


class ResourceIdPreference(TypedDict, total=False):
    """Describes the resource type and its ID preference for the user's Amazon
    Web Services account, in the current Amazon Web Services Region.
    """

    ResourceIdType: Optional[ResourceIdType]
    Resources: Optional[Resources]


class DescribeAccountPreferencesResponse(TypedDict, total=False):
    ResourceIdPreference: Optional[ResourceIdPreference]
    NextToken: Optional[Token]


class DescribeBackupPolicyRequest(ServiceRequest):
    FileSystemId: FileSystemId


class DescribeFileSystemPolicyRequest(ServiceRequest):
    FileSystemId: FileSystemId


class DescribeFileSystemsRequest(ServiceRequest):
    MaxItems: Optional[MaxItems]
    Marker: Optional[Marker]
    CreationToken: Optional[CreationToken]
    FileSystemId: Optional[FileSystemId]


FileSystemNullableSizeValue = int
Timestamp = datetime
FileSystemSizeValue = int


class FileSystemSize(TypedDict, total=False):
    """The latest known metered size (in bytes) of data stored in the file
    system, in its ``Value`` field, and the time at which that size was
    determined in its ``Timestamp`` field. The value doesn't represent the
    size of a consistent snapshot of the file system, but it is eventually
    consistent when there are no writes to the file system. That is, the
    value represents the actual size only if the file system is not modified
    for a period longer than a couple of hours. Otherwise, the value is not
    necessarily the exact size the file system was at any instant in time.
    """

    Value: FileSystemSizeValue
    Timestamp: Optional[Timestamp]
    ValueInIA: Optional[FileSystemNullableSizeValue]
    ValueInStandard: Optional[FileSystemNullableSizeValue]


class FileSystemDescription(TypedDict, total=False):
    """A description of the file system."""

    OwnerId: AwsAccountId
    CreationToken: CreationToken
    FileSystemId: FileSystemId
    FileSystemArn: Optional[FileSystemArn]
    CreationTime: Timestamp
    LifeCycleState: LifeCycleState
    Name: Optional[TagValue]
    NumberOfMountTargets: MountTargetCount
    SizeInBytes: FileSystemSize
    PerformanceMode: PerformanceMode
    Encrypted: Optional[Encrypted]
    KmsKeyId: Optional[KmsKeyId]
    ThroughputMode: Optional[ThroughputMode]
    ProvisionedThroughputInMibps: Optional[ProvisionedThroughputInMibps]
    AvailabilityZoneName: Optional[AvailabilityZoneName]
    AvailabilityZoneId: Optional[AvailabilityZoneId]
    Tags: Tags


FileSystemDescriptions = List[FileSystemDescription]


class DescribeFileSystemsResponse(TypedDict, total=False):
    Marker: Optional[Marker]
    FileSystems: Optional[FileSystemDescriptions]
    NextMarker: Optional[Marker]


class DescribeLifecycleConfigurationRequest(ServiceRequest):
    FileSystemId: FileSystemId


class DescribeMountTargetSecurityGroupsRequest(ServiceRequest):
    MountTargetId: MountTargetId


class DescribeMountTargetSecurityGroupsResponse(TypedDict, total=False):
    SecurityGroups: SecurityGroups


class DescribeMountTargetsRequest(ServiceRequest):
    MaxItems: Optional[MaxItems]
    Marker: Optional[Marker]
    FileSystemId: Optional[FileSystemId]
    MountTargetId: Optional[MountTargetId]
    AccessPointId: Optional[AccessPointId]


class MountTargetDescription(TypedDict, total=False):
    """Provides a description of a mount target."""

    OwnerId: Optional[AwsAccountId]
    MountTargetId: MountTargetId
    FileSystemId: FileSystemId
    SubnetId: SubnetId
    LifeCycleState: LifeCycleState
    IpAddress: Optional[IpAddress]
    NetworkInterfaceId: Optional[NetworkInterfaceId]
    AvailabilityZoneId: Optional[AvailabilityZoneId]
    AvailabilityZoneName: Optional[AvailabilityZoneName]
    VpcId: Optional[VpcId]


MountTargetDescriptions = List[MountTargetDescription]


class DescribeMountTargetsResponse(TypedDict, total=False):
    Marker: Optional[Marker]
    MountTargets: Optional[MountTargetDescriptions]
    NextMarker: Optional[Marker]


class DescribeReplicationConfigurationsRequest(ServiceRequest):
    FileSystemId: Optional[FileSystemId]
    NextToken: Optional[Token]
    MaxResults: Optional[MaxResults]


class Destination(TypedDict, total=False):
    """Describes the destination file system in the replication configuration."""

    Status: ReplicationStatus
    FileSystemId: FileSystemId
    Region: RegionName
    LastReplicatedTimestamp: Optional[Timestamp]


Destinations = List[Destination]


class ReplicationConfigurationDescription(TypedDict, total=False):
    SourceFileSystemId: FileSystemId
    SourceFileSystemRegion: RegionName
    SourceFileSystemArn: FileSystemArn
    OriginalSourceFileSystemArn: FileSystemArn
    CreationTime: Timestamp
    Destinations: Destinations


ReplicationConfigurationDescriptions = List[ReplicationConfigurationDescription]


class DescribeReplicationConfigurationsResponse(TypedDict, total=False):
    Replications: Optional[ReplicationConfigurationDescriptions]
    NextToken: Optional[Token]


class DescribeTagsRequest(ServiceRequest):
    MaxItems: Optional[MaxItems]
    Marker: Optional[Marker]
    FileSystemId: FileSystemId


class DescribeTagsResponse(TypedDict, total=False):
    Marker: Optional[Marker]
    Tags: Tags
    NextMarker: Optional[Marker]


class FileSystemPolicyDescription(TypedDict, total=False):
    FileSystemId: Optional[FileSystemId]
    Policy: Optional[Policy]


class LifecyclePolicy(TypedDict, total=False):
    """Describes a policy used by EFS lifecycle management and EFS
    Intelligent-Tiering that specifies when to transition files into and out
    of the file system's Infrequent Access (IA) storage class. For more
    information, see `EFS Intelligent‐Tiering and EFS Lifecycle
    Management <https://docs.aws.amazon.com/efs/latest/ug/lifecycle-management-efs.html>`__.

    When using the ``put-lifecycle-configuration`` CLI command or the
    ``PutLifecycleConfiguration`` API action, Amazon EFS requires that each
    ``LifecyclePolicy`` object have only a single transition. This means
    that in a request body, ``LifecyclePolicies`` must be structured as an
    array of ``LifecyclePolicy`` objects, one object for each transition,
    ``TransitionToIA``, ``TransitionToPrimaryStorageClass``. For more
    information, see the request examples in PutLifecycleConfiguration.
    """

    TransitionToIA: Optional[TransitionToIARules]
    TransitionToPrimaryStorageClass: Optional[TransitionToPrimaryStorageClassRules]


LifecyclePolicies = List[LifecyclePolicy]


class LifecycleConfigurationDescription(TypedDict, total=False):
    LifecyclePolicies: Optional[LifecyclePolicies]


class ListTagsForResourceRequest(ServiceRequest):
    ResourceId: ResourceId
    MaxResults: Optional[MaxResults]
    NextToken: Optional[Token]


class ListTagsForResourceResponse(TypedDict, total=False):
    Tags: Optional[Tags]
    NextToken: Optional[Token]


class ModifyMountTargetSecurityGroupsRequest(ServiceRequest):
    MountTargetId: MountTargetId
    SecurityGroups: Optional[SecurityGroups]


class PutAccountPreferencesRequest(ServiceRequest):
    ResourceIdType: ResourceIdType


class PutAccountPreferencesResponse(TypedDict, total=False):
    ResourceIdPreference: Optional[ResourceIdPreference]


class PutBackupPolicyRequest(ServiceRequest):
    FileSystemId: FileSystemId
    BackupPolicy: BackupPolicy


class PutFileSystemPolicyRequest(ServiceRequest):
    FileSystemId: FileSystemId
    Policy: Policy
    BypassPolicyLockoutSafetyCheck: Optional[BypassPolicyLockoutSafetyCheck]


class PutLifecycleConfigurationRequest(ServiceRequest):
    FileSystemId: FileSystemId
    LifecyclePolicies: LifecyclePolicies


class TagResourceRequest(ServiceRequest):
    ResourceId: ResourceId
    Tags: Tags


class UntagResourceRequest(ServiceRequest):
    ResourceId: ResourceId
    TagKeys: TagKeys


class UpdateFileSystemRequest(ServiceRequest):
    FileSystemId: FileSystemId
    ThroughputMode: Optional[ThroughputMode]
    ProvisionedThroughputInMibps: Optional[ProvisionedThroughputInMibps]


class EfsApi:

    service = "efs"
    version = "2015-02-01"

    @handler("CreateAccessPoint")
    def create_access_point(
        self,
        context: RequestContext,
        client_token: ClientToken,
        file_system_id: FileSystemId,
        tags: Tags = None,
        posix_user: PosixUser = None,
        root_directory: RootDirectory = None,
    ) -> AccessPointDescription:
        """Creates an EFS access point. An access point is an application-specific
        view into an EFS file system that applies an operating system user and
        group, and a file system path, to any file system request made through
        the access point. The operating system user and group override any
        identity information provided by the NFS client. The file system path is
        exposed as the access point's root directory. Applications using the
        access point can only access data in the application's own directory and
        any subdirectories. To learn more, see `Mounting a file system using EFS
        access
        points <https://docs.aws.amazon.com/efs/latest/ug/efs-access-points.html>`__.

        If multiple requests to create access points on the same file system are
        sent in quick succession, and the file system is near the limit of 1000
        access points, you may experience a throttling response for these
        requests. This is to ensure that the file system does not exceed the
        stated access point limit.

        This operation requires permissions for the
        ``elasticfilesystem:CreateAccessPoint`` action.

        :param client_token: A string of up to 64 ASCII characters that Amazon EFS uses to ensure
        idempotent creation.
        :param file_system_id: The ID of the EFS file system that the access point provides access to.
        :param tags: Creates tags associated with the access point.
        :param posix_user: The operating system user and group applied to all file system requests
        made using the access point.
        :param root_directory: Specifies the directory on the Amazon EFS file system that the access
        point exposes as the root directory of your file system to NFS clients
        using the access point.
        :returns: AccessPointDescription
        :raises BadRequest:
        :raises AccessPointAlreadyExists:
        :raises IncorrectFileSystemLifeCycleState:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises AccessPointLimitExceeded:
        :raises ThrottlingException:
        """
        raise NotImplementedError

    @handler("CreateFileSystem")
    def create_file_system(
        self,
        context: RequestContext,
        creation_token: CreationToken,
        performance_mode: PerformanceMode = None,
        encrypted: Encrypted = None,
        kms_key_id: KmsKeyId = None,
        throughput_mode: ThroughputMode = None,
        provisioned_throughput_in_mibps: ProvisionedThroughputInMibps = None,
        availability_zone_name: AvailabilityZoneName = None,
        backup: Backup = None,
        tags: Tags = None,
    ) -> FileSystemDescription:
        """Creates a new, empty file system. The operation requires a creation
        token in the request that Amazon EFS uses to ensure idempotent creation
        (calling the operation with same creation token has no effect). If a
        file system does not currently exist that is owned by the caller's
        Amazon Web Services account with the specified creation token, this
        operation does the following:

        -  Creates a new, empty file system. The file system will have an Amazon
           EFS assigned ID, and an initial lifecycle state ``creating``.

        -  Returns with the description of the created file system.

        Otherwise, this operation returns a ``FileSystemAlreadyExists`` error
        with the ID of the existing file system.

        For basic use cases, you can use a randomly generated UUID for the
        creation token.

        The idempotent operation allows you to retry a ``CreateFileSystem`` call
        without risk of creating an extra file system. This can happen when an
        initial call fails in a way that leaves it uncertain whether or not a
        file system was actually created. An example might be that a transport
        level timeout occurred or your connection was reset. As long as you use
        the same creation token, if the initial call had succeeded in creating a
        file system, the client can learn of its existence from the
        ``FileSystemAlreadyExists`` error.

        For more information, see `Creating a file
        system <https://docs.aws.amazon.com/efs/latest/ug/creating-using-create-fs.html#creating-using-create-fs-part1>`__
        in the *Amazon EFS User Guide*.

        The ``CreateFileSystem`` call returns while the file system's lifecycle
        state is still ``creating``. You can check the file system creation
        status by calling the DescribeFileSystems operation, which among other
        things returns the file system state.

        This operation accepts an optional ``PerformanceMode`` parameter that
        you choose for your file system. We recommend ``generalPurpose``
        performance mode for most file systems. File systems using the ``maxIO``
        performance mode can scale to higher levels of aggregate throughput and
        operations per second with a tradeoff of slightly higher latencies for
        most file operations. The performance mode can't be changed after the
        file system has been created. For more information, see `Amazon EFS
        performance
        modes <https://docs.aws.amazon.com/efs/latest/ug/performance.html#performancemodes.html>`__.

        You can set the throughput mode for the file system using the
        ``ThroughputMode`` parameter.

        After the file system is fully created, Amazon EFS sets its lifecycle
        state to ``available``, at which point you can create one or more mount
        targets for the file system in your VPC. For more information, see
        CreateMountTarget. You mount your Amazon EFS file system on an EC2
        instances in your VPC by using the mount target. For more information,
        see `Amazon EFS: How it
        Works <https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html>`__.

        This operation requires permissions for the
        ``elasticfilesystem:CreateFileSystem`` action.

        :param creation_token: A string of up to 64 ASCII characters.
        :param performance_mode: The performance mode of the file system.
        :param encrypted: A Boolean value that, if true, creates an encrypted file system.
        :param kms_key_id: The ID of the KMS key that you want to use to protect the encrypted file
        system.
        :param throughput_mode: Specifies the throughput mode for the file system.
        :param provisioned_throughput_in_mibps: The throughput, measured in MiB/s, that you want to provision for a file
        system that you're creating.
        :param availability_zone_name: Used to create a file system that uses One Zone storage classes.
        :param backup: Specifies whether automatic backups are enabled on the file system that
        you are creating.
        :param tags: Use to create one or more tags associated with the file system.
        :returns: FileSystemDescription
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemAlreadyExists:
        :raises FileSystemLimitExceeded:
        :raises InsufficientThroughputCapacity:
        :raises ThroughputLimitExceeded:
        :raises UnsupportedAvailabilityZone:
        """
        raise NotImplementedError

    @handler("CreateMountTarget")
    def create_mount_target(
        self,
        context: RequestContext,
        file_system_id: FileSystemId,
        subnet_id: SubnetId,
        ip_address: IpAddress = None,
        security_groups: SecurityGroups = None,
    ) -> MountTargetDescription:
        """Creates a mount target for a file system. You can then mount the file
        system on EC2 instances by using the mount target.

        You can create one mount target in each Availability Zone in your VPC.
        All EC2 instances in a VPC within a given Availability Zone share a
        single mount target for a given file system. If you have multiple
        subnets in an Availability Zone, you create a mount target in one of the
        subnets. EC2 instances do not need to be in the same subnet as the mount
        target in order to access their file system.

        You can create only one mount target for an EFS file system using One
        Zone storage classes. You must create that mount target in the same
        Availability Zone in which the file system is located. Use the
        ``AvailabilityZoneName`` and ``AvailabiltyZoneId`` properties in the
        DescribeFileSystems response object to get this information. Use the
        ``subnetId`` associated with the file system's Availability Zone when
        creating the mount target.

        For more information, see `Amazon EFS: How it
        Works <https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html>`__.

        To create a mount target for a file system, the file system's lifecycle
        state must be ``available``. For more information, see
        DescribeFileSystems.

        In the request, provide the following:

        -  The file system ID for which you are creating the mount target.

        -  A subnet ID, which determines the following:

           -  The VPC in which Amazon EFS creates the mount target

           -  The Availability Zone in which Amazon EFS creates the mount target

           -  The IP address range from which Amazon EFS selects the IP address
              of the mount target (if you don't specify an IP address in the
              request)

        After creating the mount target, Amazon EFS returns a response that
        includes, a ``MountTargetId`` and an ``IpAddress``. You use this IP
        address when mounting the file system in an EC2 instance. You can also
        use the mount target's DNS name when mounting the file system. The EC2
        instance on which you mount the file system by using the mount target
        can resolve the mount target's DNS name to its IP address. For more
        information, see `How it Works: Implementation
        Overview <https://docs.aws.amazon.com/efs/latest/ug/how-it-works.html#how-it-works-implementation>`__.

        Note that you can create mount targets for a file system in only one
        VPC, and there can be only one mount target per Availability Zone. That
        is, if the file system already has one or more mount targets created for
        it, the subnet specified in the request to add another mount target must
        meet the following requirements:

        -  Must belong to the same VPC as the subnets of the existing mount
           targets

        -  Must not be in the same Availability Zone as any of the subnets of
           the existing mount targets

        If the request satisfies the requirements, Amazon EFS does the
        following:

        -  Creates a new mount target in the specified subnet.

        -  Also creates a new network interface in the subnet as follows:

           -  If the request provides an ``IpAddress``, Amazon EFS assigns that
              IP address to the network interface. Otherwise, Amazon EFS assigns
              a free address in the subnet (in the same way that the Amazon EC2
              ``CreateNetworkInterface`` call does when a request does not
              specify a primary private IP address).

           -  If the request provides ``SecurityGroups``, this network interface
              is associated with those security groups. Otherwise, it belongs to
              the default security group for the subnet's VPC.

           -  Assigns the description
              ``Mount target fsmt-id for file system fs-id`` where ``fsmt-id``
              is the mount target ID, and ``fs-id`` is the ``FileSystemId``.

           -  Sets the ``requesterManaged`` property of the network interface to
              ``true``, and the ``requesterId`` value to ``EFS``.

           Each Amazon EFS mount target has one corresponding requester-managed
           EC2 network interface. After the network interface is created, Amazon
           EFS sets the ``NetworkInterfaceId`` field in the mount target's
           description to the network interface ID, and the ``IpAddress`` field
           to its address. If network interface creation fails, the entire
           ``CreateMountTarget`` operation fails.

        The ``CreateMountTarget`` call returns only after creating the network
        interface, but while the mount target state is still ``creating``, you
        can check the mount target creation status by calling the
        DescribeMountTargets operation, which among other things returns the
        mount target state.

        We recommend that you create a mount target in each of the Availability
        Zones. There are cost considerations for using a file system in an
        Availability Zone through a mount target created in another Availability
        Zone. For more information, see `Amazon
        EFS <http://aws.amazon.com/efs/>`__. In addition, by always using a
        mount target local to the instance's Availability Zone, you eliminate a
        partial failure scenario. If the Availability Zone in which your mount
        target is created goes down, then you can't access your file system
        through that mount target.

        This operation requires permissions for the following action on the file
        system:

        -  ``elasticfilesystem:CreateMountTarget``

        This operation also requires permissions for the following Amazon EC2
        actions:

        -  ``ec2:DescribeSubnets``

        -  ``ec2:DescribeNetworkInterfaces``

        -  ``ec2:CreateNetworkInterface``

        :param file_system_id: The ID of the file system for which to create the mount target.
        :param subnet_id: The ID of the subnet to add the mount target in.
        :param ip_address: Valid IPv4 address within the address range of the specified subnet.
        :param security_groups: Up to five VPC security group IDs, of the form ``sg-xxxxxxxx``.
        :returns: MountTargetDescription
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises IncorrectFileSystemLifeCycleState:
        :raises MountTargetConflict:
        :raises SubnetNotFound:
        :raises NoFreeAddressesInSubnet:
        :raises IpAddressInUse:
        :raises NetworkInterfaceLimitExceeded:
        :raises SecurityGroupLimitExceeded:
        :raises SecurityGroupNotFound:
        :raises UnsupportedAvailabilityZone:
        :raises AvailabilityZonesMismatch:
        """
        raise NotImplementedError

    @handler("CreateReplicationConfiguration")
    def create_replication_configuration(
        self,
        context: RequestContext,
        source_file_system_id: FileSystemId,
        destinations: DestinationsToCreate,
    ) -> ReplicationConfigurationDescription:
        """Creates a replication configuration that replicates an existing EFS file
        system to a new, read-only file system. For more information, see
        `Amazon EFS
        replication <https://docs.aws.amazon.com/efs/latest/ug/efs-replication.html>`__
        in the *Amazon EFS User Guide*. The replication configuration specifies
        the following:

        -  **Source file system** - An existing EFS file system that you want
           replicated. The source file system cannot be a destination file
           system in an existing replication configuration.

        -  **Destination file system configuration** - The configuration of the
           destination file system to which the source file system will be
           replicated. There can only be one destination file system in a
           replication configuration. The destination file system configuration
           consists of the following properties:

           -  **Amazon Web Services Region** - The Amazon Web Services Region in
              which the destination file system is created. Amazon EFS
              replication is available in all Amazon Web Services Regions that
              Amazon EFS is available in, except Africa (Cape Town), Asia
              Pacific (Hong Kong), Asia Pacific (Jakarta), Europe (Milan), and
              Middle East (Bahrain).

           -  **Availability Zone** - If you want the destination file system to
              use EFS One Zone availability and durability, you must specify the
              Availability Zone to create the file system in. For more
              information about EFS storage classes, see `Amazon EFS storage
              classes <https://docs.aws.amazon.com/efs/latest/ug/storage-classes.html>`__
              in the *Amazon EFS User Guide*.

           -  **Encryption** - All destination file systems are created with
              encryption at rest enabled. You can specify the Key Management
              Service (KMS) key that is used to encrypt the destination file
              system. If you don't specify a KMS key, your service-managed KMS
              key for Amazon EFS is used.

              After the file system is created, you cannot change the KMS key.

        The following properties are set by default:

        -  **Performance mode** - The destination file system's performance mode
           matches that of the source file system, unless the destination file
           system uses EFS One Zone storage. In that case, the General Purpose
           performance mode is used. The performance mode cannot be changed.

        -  **Throughput mode** - The destination file system uses the Bursting
           Throughput mode by default. After the file system is created, you can
           modify the throughput mode.

        The following properties are turned off by default:

        -  **Lifecycle management** - EFS lifecycle management and EFS
           Intelligent-Tiering are not enabled on the destination file system.
           After the destination file system is created, you can enable EFS
           lifecycle management and EFS Intelligent-Tiering.

        -  **Automatic backups** - Automatic daily backups not enabled on the
           destination file system. After the file system is created, you can
           change this setting.

        For more information, see `Amazon EFS
        replication <https://docs.aws.amazon.com/efs/latest/ug/efs-replication.html>`__
        in the *Amazon EFS User Guide*.

        :param source_file_system_id: Specifies the Amazon EFS file system that you want to replicate.
        :param destinations: An array of destination configuration objects.
        :returns: ReplicationConfigurationDescription
        :raises BadRequest:
        :raises IncorrectFileSystemLifeCycleState:
        :raises ValidationException:
        :raises ReplicationNotFound:
        :raises FileSystemNotFound:
        :raises UnsupportedAvailabilityZone:
        :raises FileSystemLimitExceeded:
        :raises InsufficientThroughputCapacity:
        :raises ThroughputLimitExceeded:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("CreateTags")
    def create_tags(
        self, context: RequestContext, file_system_id: FileSystemId, tags: Tags
    ) -> None:
        """DEPRECATED - ``CreateTags`` is deprecated and not maintained. To create
        tags for EFS resources, use the API action.

        Creates or overwrites tags associated with a file system. Each tag is a
        key-value pair. If a tag key specified in the request already exists on
        the file system, this operation overwrites its value with the value
        provided in the request. If you add the ``Name`` tag to your file
        system, Amazon EFS returns it in the response to the DescribeFileSystems
        operation.

        This operation requires permission for the
        ``elasticfilesystem:CreateTags`` action.

        :param file_system_id: The ID of the file system whose tags you want to modify (String).
        :param tags: An array of ``Tag`` objects to add.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        """
        raise NotImplementedError

    @handler("DeleteAccessPoint")
    def delete_access_point(self, context: RequestContext, access_point_id: AccessPointId) -> None:
        """Deletes the specified access point. After deletion is complete, new
        clients can no longer connect to the access points. Clients connected to
        the access point at the time of deletion will continue to function until
        they terminate their connection.

        This operation requires permissions for the
        ``elasticfilesystem:DeleteAccessPoint`` action.

        :param access_point_id: The ID of the access point that you want to delete.
        :raises BadRequest:
        :raises InternalServerError:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("DeleteFileSystem")
    def delete_file_system(self, context: RequestContext, file_system_id: FileSystemId) -> None:
        """Deletes a file system, permanently severing access to its contents. Upon
        return, the file system no longer exists and you can't access any
        contents of the deleted file system.

        You need to manually delete mount targets attached to a file system
        before you can delete an EFS file system. This step is performed for you
        when you use the Amazon Web Services console to delete a file system.

        You cannot delete a file system that is part of an EFS Replication
        configuration. You need to delete the replication configuration first.

        You can't delete a file system that is in use. That is, if the file
        system has any mount targets, you must first delete them. For more
        information, see DescribeMountTargets and DeleteMountTarget.

        The ``DeleteFileSystem`` call returns while the file system state is
        still ``deleting``. You can check the file system deletion status by
        calling the DescribeFileSystems operation, which returns a list of file
        systems in your account. If you pass file system ID or creation token
        for the deleted file system, the DescribeFileSystems returns a
        ``404 FileSystemNotFound`` error.

        This operation requires permissions for the
        ``elasticfilesystem:DeleteFileSystem`` action.

        :param file_system_id: The ID of the file system you want to delete.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises FileSystemInUse:
        """
        raise NotImplementedError

    @handler("DeleteFileSystemPolicy")
    def delete_file_system_policy(
        self, context: RequestContext, file_system_id: FileSystemId
    ) -> None:
        """Deletes the ``FileSystemPolicy`` for the specified file system. The
        default ``FileSystemPolicy`` goes into effect once the existing policy
        is deleted. For more information about the default file system policy,
        see `Using Resource-based Policies with
        EFS <https://docs.aws.amazon.com/efs/latest/ug/res-based-policies-efs.html>`__.

        This operation requires permissions for the
        ``elasticfilesystem:DeleteFileSystemPolicy`` action.

        :param file_system_id: Specifies the EFS file system for which to delete the
        ``FileSystemPolicy``.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises IncorrectFileSystemLifeCycleState:
        """
        raise NotImplementedError

    @handler("DeleteMountTarget")
    def delete_mount_target(self, context: RequestContext, mount_target_id: MountTargetId) -> None:
        """Deletes the specified mount target.

        This operation forcibly breaks any mounts of the file system by using
        the mount target that is being deleted, which might disrupt instances or
        applications using those mounts. To avoid applications getting cut off
        abruptly, you might consider unmounting any mounts of the mount target,
        if feasible. The operation also deletes the associated network
        interface. Uncommitted writes might be lost, but breaking a mount target
        using this operation does not corrupt the file system itself. The file
        system you created remains. You can mount an EC2 instance in your VPC by
        using another mount target.

        This operation requires permissions for the following action on the file
        system:

        -  ``elasticfilesystem:DeleteMountTarget``

        The ``DeleteMountTarget`` call returns while the mount target state is
        still ``deleting``. You can check the mount target deletion by calling
        the DescribeMountTargets operation, which returns a list of mount target
        descriptions for the given file system.

        The operation also requires permissions for the following Amazon EC2
        action on the mount target's network interface:

        -  ``ec2:DeleteNetworkInterface``

        :param mount_target_id: The ID of the mount target to delete (String).
        :raises BadRequest:
        :raises InternalServerError:
        :raises DependencyTimeout:
        :raises MountTargetNotFound:
        """
        raise NotImplementedError

    @handler("DeleteReplicationConfiguration")
    def delete_replication_configuration(
        self, context: RequestContext, source_file_system_id: FileSystemId
    ) -> None:
        """Deletes an existing replication configuration. To delete a replication
        configuration, you must make the request from the Amazon Web Services
        Region in which the destination file system is located. Deleting a
        replication configuration ends the replication process. After a
        replication configuration is deleted, the destination file system is no
        longer read-only. You can write to the destination file system after its
        status becomes ``Writeable``.

        :param source_file_system_id: The ID of the source file system in the replication configuration.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises ReplicationNotFound:
        """
        raise NotImplementedError

    @handler("DeleteTags")
    def delete_tags(
        self, context: RequestContext, file_system_id: FileSystemId, tag_keys: TagKeys
    ) -> None:
        """DEPRECATED - ``DeleteTags`` is deprecated and not maintained. To remove
        tags from EFS resources, use the API action.

        Deletes the specified tags from a file system. If the ``DeleteTags``
        request includes a tag key that doesn't exist, Amazon EFS ignores it and
        doesn't cause an error. For more information about tags and related
        restrictions, see `Tag
        restrictions <https://docs.aws.amazon.com/awsaccountbilling/latest/aboutv2/cost-alloc-tags.html>`__
        in the *Billing and Cost Management User Guide*.

        This operation requires permissions for the
        ``elasticfilesystem:DeleteTags`` action.

        :param file_system_id: The ID of the file system whose tags you want to delete (String).
        :param tag_keys: A list of tag keys to delete.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        """
        raise NotImplementedError

    @handler("DescribeAccessPoints")
    def describe_access_points(
        self,
        context: RequestContext,
        max_results: MaxResults = None,
        next_token: Token = None,
        access_point_id: AccessPointId = None,
        file_system_id: FileSystemId = None,
    ) -> DescribeAccessPointsResponse:
        """Returns the description of a specific Amazon EFS access point if the
        ``AccessPointId`` is provided. If you provide an EFS ``FileSystemId``,
        it returns descriptions of all access points for that file system. You
        can provide either an ``AccessPointId`` or a ``FileSystemId`` in the
        request, but not both.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeAccessPoints`` action.

        :param max_results: (Optional) When retrieving all access points for a file system, you can
        optionally specify the ``MaxItems`` parameter to limit the number of
        objects returned in a response.
        :param next_token: ``NextToken`` is present if the response is paginated.
        :param access_point_id: (Optional) Specifies an EFS access point to describe in the response;
        mutually exclusive with ``FileSystemId``.
        :param file_system_id: (Optional) If you provide a ``FileSystemId``, EFS returns all access
        points for that file system; mutually exclusive with ``AccessPointId``.
        :returns: DescribeAccessPointsResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("DescribeAccountPreferences")
    def describe_account_preferences(
        self, context: RequestContext, next_token: Token = None, max_results: MaxResults = None
    ) -> DescribeAccountPreferencesResponse:
        """Returns the account preferences settings for the Amazon Web Services
        account associated with the user making the request, in the current
        Amazon Web Services Region. For more information, see `Managing Amazon
        EFS resource IDs <efs/latest/ug/manage-efs-resource-ids.html>`__.

        :param next_token: (Optional) You can use ``NextToken`` in a subsequent request to fetch
        the next page of Amazon Web Services account preferences if the response
        payload was paginated.
        :param max_results: (Optional) When retrieving account preferences, you can optionally
        specify the ``MaxItems`` parameter to limit the number of objects
        returned in a response.
        :returns: DescribeAccountPreferencesResponse
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("DescribeBackupPolicy")
    def describe_backup_policy(
        self, context: RequestContext, file_system_id: FileSystemId
    ) -> BackupPolicyDescription:
        """Returns the backup policy for the specified EFS file system.

        :param file_system_id: Specifies which EFS file system to retrieve the ``BackupPolicy`` for.
        :returns: BackupPolicyDescription
        :raises BadRequest:
        :raises FileSystemNotFound:
        :raises InternalServerError:
        :raises PolicyNotFound:
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("DescribeFileSystemPolicy")
    def describe_file_system_policy(
        self, context: RequestContext, file_system_id: FileSystemId
    ) -> FileSystemPolicyDescription:
        """Returns the ``FileSystemPolicy`` for the specified EFS file system.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeFileSystemPolicy`` action.

        :param file_system_id: Specifies which EFS file system to retrieve the ``FileSystemPolicy``
        for.
        :returns: FileSystemPolicyDescription
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises PolicyNotFound:
        """
        raise NotImplementedError

    @handler("DescribeFileSystems")
    def describe_file_systems(
        self,
        context: RequestContext,
        max_items: MaxItems = None,
        marker: Marker = None,
        creation_token: CreationToken = None,
        file_system_id: FileSystemId = None,
    ) -> DescribeFileSystemsResponse:
        """Returns the description of a specific Amazon EFS file system if either
        the file system ``CreationToken`` or the ``FileSystemId`` is provided.
        Otherwise, it returns descriptions of all file systems owned by the
        caller's Amazon Web Services account in the Amazon Web Services Region
        of the endpoint that you're calling.

        When retrieving all file system descriptions, you can optionally specify
        the ``MaxItems`` parameter to limit the number of descriptions in a
        response. This number is automatically set to 100. If more file system
        descriptions remain, Amazon EFS returns a ``NextMarker``, an opaque
        token, in the response. In this case, you should send a subsequent
        request with the ``Marker`` request parameter set to the value of
        ``NextMarker``.

        To retrieve a list of your file system descriptions, this operation is
        used in an iterative process, where ``DescribeFileSystems`` is called
        first without the ``Marker`` and then the operation continues to call it
        with the ``Marker`` parameter set to the value of the ``NextMarker``
        from the previous response until the response has no ``NextMarker``.

        The order of file systems returned in the response of one
        ``DescribeFileSystems`` call and the order of file systems returned
        across the responses of a multi-call iteration is unspecified.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeFileSystems`` action.

        :param max_items: (Optional) Specifies the maximum number of file systems to return in the
        response (integer).
        :param marker: (Optional) Opaque pagination token returned from a previous
        ``DescribeFileSystems`` operation (String).
        :param creation_token: (Optional) Restricts the list to the file system with this creation
        token (String).
        :param file_system_id: (Optional) ID of the file system whose description you want to retrieve
        (String).
        :returns: DescribeFileSystemsResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        """
        raise NotImplementedError

    @handler("DescribeLifecycleConfiguration")
    def describe_lifecycle_configuration(
        self, context: RequestContext, file_system_id: FileSystemId
    ) -> LifecycleConfigurationDescription:
        """Returns the current ``LifecycleConfiguration`` object for the specified
        Amazon EFS file system. EFS lifecycle management uses the
        ``LifecycleConfiguration`` object to identify which files to move to the
        EFS Infrequent Access (IA) storage class. For a file system without a
        ``LifecycleConfiguration`` object, the call returns an empty array in
        the response.

        When EFS Intelligent-Tiering is enabled,
        ``TransitionToPrimaryStorageClass`` has a value of ``AFTER_1_ACCESS``.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeLifecycleConfiguration`` operation.

        :param file_system_id: The ID of the file system whose ``LifecycleConfiguration`` object you
        want to retrieve (String).
        :returns: LifecycleConfigurationDescription
        :raises InternalServerError:
        :raises BadRequest:
        :raises FileSystemNotFound:
        """
        raise NotImplementedError

    @handler("DescribeMountTargetSecurityGroups")
    def describe_mount_target_security_groups(
        self, context: RequestContext, mount_target_id: MountTargetId
    ) -> DescribeMountTargetSecurityGroupsResponse:
        """Returns the security groups currently in effect for a mount target. This
        operation requires that the network interface of the mount target has
        been created and the lifecycle state of the mount target is not
        ``deleted``.

        This operation requires permissions for the following actions:

        -  ``elasticfilesystem:DescribeMountTargetSecurityGroups`` action on the
           mount target's file system.

        -  ``ec2:DescribeNetworkInterfaceAttribute`` action on the mount
           target's network interface.

        :param mount_target_id: The ID of the mount target whose security groups you want to retrieve.
        :returns: DescribeMountTargetSecurityGroupsResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises MountTargetNotFound:
        :raises IncorrectMountTargetState:
        """
        raise NotImplementedError

    @handler("DescribeMountTargets")
    def describe_mount_targets(
        self,
        context: RequestContext,
        max_items: MaxItems = None,
        marker: Marker = None,
        file_system_id: FileSystemId = None,
        mount_target_id: MountTargetId = None,
        access_point_id: AccessPointId = None,
    ) -> DescribeMountTargetsResponse:
        """Returns the descriptions of all the current mount targets, or a specific
        mount target, for a file system. When requesting all of the current
        mount targets, the order of mount targets returned in the response is
        unspecified.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeMountTargets`` action, on either the file
        system ID that you specify in ``FileSystemId``, or on the file system of
        the mount target that you specify in ``MountTargetId``.

        :param max_items: (Optional) Maximum number of mount targets to return in the response.
        :param marker: (Optional) Opaque pagination token returned from a previous
        ``DescribeMountTargets`` operation (String).
        :param file_system_id: (Optional) ID of the file system whose mount targets you want to list
        (String).
        :param mount_target_id: (Optional) ID of the mount target that you want to have described
        (String).
        :param access_point_id: (Optional) The ID of the access point whose mount targets that you want
        to list.
        :returns: DescribeMountTargetsResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises MountTargetNotFound:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("DescribeReplicationConfigurations")
    def describe_replication_configurations(
        self,
        context: RequestContext,
        file_system_id: FileSystemId = None,
        next_token: Token = None,
        max_results: MaxResults = None,
    ) -> DescribeReplicationConfigurationsResponse:
        """Retrieves the replication configuration for a specific file system. If a
        file system is not specified, all of the replication configurations for
        the Amazon Web Services account in an Amazon Web Services Region are
        retrieved.

        :param file_system_id: You can retrieve the replication configuration for a specific file
        system by providing its file system ID.
        :param next_token: ``NextToken`` is present if the response is paginated.
        :param max_results: (Optional) To limit the number of objects returned in a response, you
        can specify the ``MaxItems`` parameter.
        :returns: DescribeReplicationConfigurationsResponse
        :raises BadRequest:
        :raises FileSystemNotFound:
        :raises InternalServerError:
        :raises ReplicationNotFound:
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("DescribeTags")
    def describe_tags(
        self,
        context: RequestContext,
        file_system_id: FileSystemId,
        max_items: MaxItems = None,
        marker: Marker = None,
    ) -> DescribeTagsResponse:
        """DEPRECATED - The ``DescribeTags`` action is deprecated and not
        maintained. To view tags associated with EFS resources, use the
        ``ListTagsForResource`` API action.

        Returns the tags associated with a file system. The order of tags
        returned in the response of one ``DescribeTags`` call and the order of
        tags returned across the responses of a multiple-call iteration (when
        using pagination) is unspecified.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeTags`` action.

        :param file_system_id: The ID of the file system whose tag set you want to retrieve.
        :param max_items: (Optional) The maximum number of file system tags to return in the
        response.
        :param marker: (Optional) An opaque pagination token returned from a previous
        ``DescribeTags`` operation (String).
        :returns: DescribeTagsResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self,
        context: RequestContext,
        resource_id: ResourceId,
        max_results: MaxResults = None,
        next_token: Token = None,
    ) -> ListTagsForResourceResponse:
        """Lists all tags for a top-level EFS resource. You must provide the ID of
        the resource that you want to retrieve the tags for.

        This operation requires permissions for the
        ``elasticfilesystem:DescribeAccessPoints`` action.

        :param resource_id: Specifies the EFS resource you want to retrieve tags for.
        :param max_results: (Optional) Specifies the maximum number of tag objects to return in the
        response.
        :param next_token: (Optional) You can use ``NextToken`` in a subsequent request to fetch
        the next page of access point descriptions if the response payload was
        paginated.
        :returns: ListTagsForResourceResponse
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("ModifyMountTargetSecurityGroups")
    def modify_mount_target_security_groups(
        self,
        context: RequestContext,
        mount_target_id: MountTargetId,
        security_groups: SecurityGroups = None,
    ) -> None:
        """Modifies the set of security groups in effect for a mount target.

        When you create a mount target, Amazon EFS also creates a new network
        interface. For more information, see CreateMountTarget. This operation
        replaces the security groups in effect for the network interface
        associated with a mount target, with the ``SecurityGroups`` provided in
        the request. This operation requires that the network interface of the
        mount target has been created and the lifecycle state of the mount
        target is not ``deleted``.

        The operation requires permissions for the following actions:

        -  ``elasticfilesystem:ModifyMountTargetSecurityGroups`` action on the
           mount target's file system.

        -  ``ec2:ModifyNetworkInterfaceAttribute`` action on the mount target's
           network interface.

        :param mount_target_id: The ID of the mount target whose security groups you want to modify.
        :param security_groups: An array of up to five VPC security group IDs.
        :raises BadRequest:
        :raises InternalServerError:
        :raises MountTargetNotFound:
        :raises IncorrectMountTargetState:
        :raises SecurityGroupLimitExceeded:
        :raises SecurityGroupNotFound:
        """
        raise NotImplementedError

    @handler("PutAccountPreferences")
    def put_account_preferences(
        self, context: RequestContext, resource_id_type: ResourceIdType
    ) -> PutAccountPreferencesResponse:
        """Use this operation to set the account preference in the current Amazon
        Web Services Region to use long 17 character (63 bit) or short 8
        character (32 bit) resource IDs for new EFS file system and mount target
        resources. All existing resource IDs are not affected by any changes you
        make. You can set the ID preference during the opt-in period as EFS
        transitions to long resource IDs. For more information, see `Managing
        Amazon EFS resource
        IDs <https://docs.aws.amazon.com/efs/latest/ug/manage-efs-resource-ids.html>`__.

        Starting in October, 2021, you will receive an error if you try to set
        the account preference to use the short 8 character format resource ID.
        Contact Amazon Web Services support if you receive an error and must use
        short IDs for file system and mount target resources.

        :param resource_id_type: Specifies the EFS resource ID preference to set for the user's Amazon
        Web Services account, in the current Amazon Web Services Region, either
        ``LONG_ID`` (17 characters), or ``SHORT_ID`` (8 characters).
        :returns: PutAccountPreferencesResponse
        :raises BadRequest:
        :raises InternalServerError:
        """
        raise NotImplementedError

    @handler("PutBackupPolicy")
    def put_backup_policy(
        self, context: RequestContext, file_system_id: FileSystemId, backup_policy: BackupPolicy
    ) -> BackupPolicyDescription:
        """Updates the file system's backup policy. Use this action to start or
        stop automatic backups of the file system.

        :param file_system_id: Specifies which EFS file system to update the backup policy for.
        :param backup_policy: The backup policy included in the ``PutBackupPolicy`` request.
        :returns: BackupPolicyDescription
        :raises BadRequest:
        :raises FileSystemNotFound:
        :raises IncorrectFileSystemLifeCycleState:
        :raises InternalServerError:
        :raises ValidationException:
        """
        raise NotImplementedError

    @handler("PutFileSystemPolicy")
    def put_file_system_policy(
        self,
        context: RequestContext,
        file_system_id: FileSystemId,
        policy: Policy,
        bypass_policy_lockout_safety_check: BypassPolicyLockoutSafetyCheck = None,
    ) -> FileSystemPolicyDescription:
        """Applies an Amazon EFS ``FileSystemPolicy`` to an Amazon EFS file system.
        A file system policy is an IAM resource-based policy and can contain
        multiple policy statements. A file system always has exactly one file
        system policy, which can be the default policy or an explicit policy set
        or updated using this API operation. EFS file system policies have a
        20,000 character limit. When an explicit policy is set, it overrides the
        default policy. For more information about the default file system
        policy, see `Default EFS File System
        Policy <https://docs.aws.amazon.com/efs/latest/ug/iam-access-control-nfs-efs.html#default-filesystempolicy>`__.

        EFS file system policies have a 20,000 character limit.

        This operation requires permissions for the
        ``elasticfilesystem:PutFileSystemPolicy`` action.

        :param file_system_id: The ID of the EFS file system that you want to create or update the
        ``FileSystemPolicy`` for.
        :param policy: The ``FileSystemPolicy`` that you're creating.
        :param bypass_policy_lockout_safety_check: (Optional) A boolean that specifies whether or not to bypass the
        ``FileSystemPolicy`` lockout safety check.
        :returns: FileSystemPolicyDescription
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises InvalidPolicyException:
        :raises IncorrectFileSystemLifeCycleState:
        """
        raise NotImplementedError

    @handler("PutLifecycleConfiguration")
    def put_lifecycle_configuration(
        self,
        context: RequestContext,
        file_system_id: FileSystemId,
        lifecycle_policies: LifecyclePolicies,
    ) -> LifecycleConfigurationDescription:
        """Use this action to manage EFS lifecycle management and EFS
        Intelligent-Tiering. A ``LifecycleConfiguration`` consists of one or
        more ``LifecyclePolicy`` objects that define the following:

        -  **EFS Lifecycle management** - When Amazon EFS automatically
           transitions files in a file system into the lower-cost EFS Infrequent
           Access (IA) storage class.

           To enable EFS Lifecycle management, set the value of
           ``TransitionToIA`` to one of the available options.

        -  **EFS Intelligent-Tiering** - When Amazon EFS automatically
           transitions files from IA back into the file system's primary storage
           class (EFS Standard or EFS One Zone Standard).

           To enable EFS Intelligent-Tiering, set the value of
           ``TransitionToPrimaryStorageClass`` to ``AFTER_1_ACCESS``.

        For more information, see `EFS Lifecycle
        Management <https://docs.aws.amazon.com/efs/latest/ug/lifecycle-management-efs.html>`__.

        Each Amazon EFS file system supports one lifecycle configuration, which
        applies to all files in the file system. If a ``LifecycleConfiguration``
        object already exists for the specified file system, a
        ``PutLifecycleConfiguration`` call modifies the existing configuration.
        A ``PutLifecycleConfiguration`` call with an empty ``LifecyclePolicies``
        array in the request body deletes any existing
        ``LifecycleConfiguration`` and turns off lifecycle management and EFS
        Intelligent-Tiering for the file system.

        In the request, specify the following:

        -  The ID for the file system for which you are enabling, disabling, or
           modifying lifecycle management and EFS Intelligent-Tiering.

        -  A ``LifecyclePolicies`` array of ``LifecyclePolicy`` objects that
           define when files are moved into IA storage, and when they are moved
           back to Standard storage.

           Amazon EFS requires that each ``LifecyclePolicy`` object have only
           have a single transition, so the ``LifecyclePolicies`` array needs to
           be structured with separate ``LifecyclePolicy`` objects. See the
           example requests in the following section for more information.

        This operation requires permissions for the
        ``elasticfilesystem:PutLifecycleConfiguration`` operation.

        To apply a ``LifecycleConfiguration`` object to an encrypted file
        system, you need the same Key Management Service permissions as when you
        created the encrypted file system.

        :param file_system_id: The ID of the file system for which you are creating the
        ``LifecycleConfiguration`` object (String).
        :param lifecycle_policies: An array of ``LifecyclePolicy`` objects that define the file system's
        ``LifecycleConfiguration`` object.
        :returns: LifecycleConfigurationDescription
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises IncorrectFileSystemLifeCycleState:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(self, context: RequestContext, resource_id: ResourceId, tags: Tags) -> None:
        """Creates a tag for an EFS resource. You can create tags for EFS file
        systems and access points using this API operation.

        This operation requires permissions for the
        ``elasticfilesystem:TagResource`` action.

        :param resource_id: The ID specifying the EFS resource that you want to create a tag for.
        :param tags: An array of ``Tag`` objects to add.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_id: ResourceId, tag_keys: TagKeys
    ) -> None:
        """Removes tags from an EFS resource. You can remove tags from EFS file
        systems and access points using this API operation.

        This operation requires permissions for the
        ``elasticfilesystem:UntagResource`` action.

        :param resource_id: Specifies the EFS resource that you want to remove tags from.
        :param tag_keys: The keys of the key-value tag pairs that you want to remove from the
        specified EFS resource.
        :raises BadRequest:
        :raises InternalServerError:
        :raises FileSystemNotFound:
        :raises AccessPointNotFound:
        """
        raise NotImplementedError

    @handler("UpdateFileSystem")
    def update_file_system(
        self,
        context: RequestContext,
        file_system_id: FileSystemId,
        throughput_mode: ThroughputMode = None,
        provisioned_throughput_in_mibps: ProvisionedThroughputInMibps = None,
    ) -> FileSystemDescription:
        """Updates the throughput mode or the amount of provisioned throughput of
        an existing file system.

        :param file_system_id: The ID of the file system that you want to update.
        :param throughput_mode: (Optional) Updates the file system's throughput mode.
        :param provisioned_throughput_in_mibps: (Optional) Sets the amount of provisioned throughput, in MiB/s, for the
        file system.
        :returns: FileSystemDescription
        :raises BadRequest:
        :raises FileSystemNotFound:
        :raises IncorrectFileSystemLifeCycleState:
        :raises InsufficientThroughputCapacity:
        :raises InternalServerError:
        :raises ThroughputLimitExceeded:
        :raises TooManyRequests:
        """
        raise NotImplementedError



import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ARNString = str
AccessKeyString = str
AccountId = str
ClaimName = str
ClaimValue = str
ClassicFlow = bool
CognitoIdentityProviderClientId = str
CognitoIdentityProviderName = str
CognitoIdentityProviderTokenCheck = bool
DeveloperProviderName = str
DeveloperUserIdentifier = str
HideDisabled = bool
IdentityId = str
IdentityPoolId = str
IdentityPoolName = str
IdentityPoolUnauthenticated = bool
IdentityProviderId = str
IdentityProviderName = str
IdentityProviderToken = str
OIDCToken = str
PaginationKey = str
PrincipalTagID = str
PrincipalTagValue = str
QueryLimit = int
RoleType = str
SecretKeyString = str
SessionTokenString = str
String = str
TagKeysType = str
TagValueType = str
UseDefaults = bool


class AmbiguousRoleResolutionType(str):
    AuthenticatedRole = "AuthenticatedRole"
    Deny = "Deny"


class ErrorCode(str):
    AccessDenied = "AccessDenied"
    InternalServerError = "InternalServerError"


class MappingRuleMatchType(str):
    Equals = "Equals"
    Contains = "Contains"
    StartsWith = "StartsWith"
    NotEqual = "NotEqual"


class RoleMappingType(str):
    Token = "Token"
    Rules = "Rules"


class ConcurrentModificationException(ServiceException):
    """Thrown if there are parallel requests to modify a resource."""

    code: str = "ConcurrentModificationException"
    sender_fault: bool = False
    status_code: int = 400


class DeveloperUserAlreadyRegisteredException(ServiceException):
    """The provided developer user identifier is already registered with
    Cognito under a different identity ID.
    """

    code: str = "DeveloperUserAlreadyRegisteredException"
    sender_fault: bool = False
    status_code: int = 400


class ExternalServiceException(ServiceException):
    """An exception thrown when a dependent service such as Facebook or Twitter
    is not responding
    """

    code: str = "ExternalServiceException"
    sender_fault: bool = False
    status_code: int = 400


class InternalErrorException(ServiceException):
    """Thrown when the service encounters an error during processing the
    request.
    """

    code: str = "InternalErrorException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidIdentityPoolConfigurationException(ServiceException):
    """Thrown if the identity pool has no role associated for the given auth
    type (auth/unauth) or if the AssumeRole fails.
    """

    code: str = "InvalidIdentityPoolConfigurationException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidParameterException(ServiceException):
    """Thrown for missing or bad input parameter(s)."""

    code: str = "InvalidParameterException"
    sender_fault: bool = False
    status_code: int = 400


class LimitExceededException(ServiceException):
    """Thrown when the total number of user pools has exceeded a preset limit."""

    code: str = "LimitExceededException"
    sender_fault: bool = False
    status_code: int = 400


class NotAuthorizedException(ServiceException):
    """Thrown when a user is not authorized to access the requested resource."""

    code: str = "NotAuthorizedException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceConflictException(ServiceException):
    """Thrown when a user tries to use a login which is already linked to
    another account.
    """

    code: str = "ResourceConflictException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceNotFoundException(ServiceException):
    """Thrown when the requested resource (for example, a dataset or record)
    does not exist.
    """

    code: str = "ResourceNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


class TooManyRequestsException(ServiceException):
    """Thrown when a request is throttled."""

    code: str = "TooManyRequestsException"
    sender_fault: bool = False
    status_code: int = 400


class CognitoIdentityProvider(TypedDict, total=False):
    """A provider representing an Amazon Cognito user pool and its client ID."""

    ProviderName: Optional[CognitoIdentityProviderName]
    ClientId: Optional[CognitoIdentityProviderClientId]
    ServerSideTokenCheck: Optional[CognitoIdentityProviderTokenCheck]


CognitoIdentityProviderList = List[CognitoIdentityProvider]
IdentityPoolTagsType = Dict[TagKeysType, TagValueType]
SAMLProviderList = List[ARNString]
OIDCProviderList = List[ARNString]
IdentityProviders = Dict[IdentityProviderName, IdentityProviderId]


class CreateIdentityPoolInput(ServiceRequest):
    """Input to the CreateIdentityPool action."""

    IdentityPoolName: IdentityPoolName
    AllowUnauthenticatedIdentities: IdentityPoolUnauthenticated
    AllowClassicFlow: Optional[ClassicFlow]
    SupportedLoginProviders: Optional[IdentityProviders]
    DeveloperProviderName: Optional[DeveloperProviderName]
    OpenIdConnectProviderARNs: Optional[OIDCProviderList]
    CognitoIdentityProviders: Optional[CognitoIdentityProviderList]
    SamlProviderARNs: Optional[SAMLProviderList]
    IdentityPoolTags: Optional[IdentityPoolTagsType]


DateType = datetime


class Credentials(TypedDict, total=False):
    """Credentials for the provided identity ID."""

    AccessKeyId: Optional[AccessKeyString]
    SecretKey: Optional[SecretKeyString]
    SessionToken: Optional[SessionTokenString]
    Expiration: Optional[DateType]


IdentityIdList = List[IdentityId]


class DeleteIdentitiesInput(ServiceRequest):
    """Input to the ``DeleteIdentities`` action."""

    IdentityIdsToDelete: IdentityIdList


class UnprocessedIdentityId(TypedDict, total=False):
    """An array of UnprocessedIdentityId objects, each of which contains an
    ErrorCode and IdentityId.
    """

    IdentityId: Optional[IdentityId]
    ErrorCode: Optional[ErrorCode]


UnprocessedIdentityIdList = List[UnprocessedIdentityId]


class DeleteIdentitiesResponse(TypedDict, total=False):
    """Returned in response to a successful ``DeleteIdentities`` operation."""

    UnprocessedIdentityIds: Optional[UnprocessedIdentityIdList]


class DeleteIdentityPoolInput(ServiceRequest):
    """Input to the DeleteIdentityPool action."""

    IdentityPoolId: IdentityPoolId


class DescribeIdentityInput(ServiceRequest):
    """Input to the ``DescribeIdentity`` action."""

    IdentityId: IdentityId


class DescribeIdentityPoolInput(ServiceRequest):
    """Input to the DescribeIdentityPool action."""

    IdentityPoolId: IdentityPoolId


DeveloperUserIdentifierList = List[DeveloperUserIdentifier]
LoginsMap = Dict[IdentityProviderName, IdentityProviderToken]


class GetCredentialsForIdentityInput(ServiceRequest):
    """Input to the ``GetCredentialsForIdentity`` action."""

    IdentityId: IdentityId
    Logins: Optional[LoginsMap]
    CustomRoleArn: Optional[ARNString]


class GetCredentialsForIdentityResponse(TypedDict, total=False):
    """Returned in response to a successful ``GetCredentialsForIdentity``
    operation.
    """

    IdentityId: Optional[IdentityId]
    Credentials: Optional[Credentials]


class GetIdInput(ServiceRequest):
    """Input to the GetId action."""

    AccountId: Optional[AccountId]
    IdentityPoolId: IdentityPoolId
    Logins: Optional[LoginsMap]


class GetIdResponse(TypedDict, total=False):
    """Returned in response to a GetId request."""

    IdentityId: Optional[IdentityId]


class GetIdentityPoolRolesInput(ServiceRequest):
    """Input to the ``GetIdentityPoolRoles`` action."""

    IdentityPoolId: IdentityPoolId


class MappingRule(TypedDict, total=False):
    """A rule that maps a claim name, a claim value, and a match type to a role
    ARN.
    """

    Claim: ClaimName
    MatchType: MappingRuleMatchType
    Value: ClaimValue
    RoleARN: ARNString


MappingRulesList = List[MappingRule]


class RulesConfigurationType(TypedDict, total=False):
    """A container for rules."""

    Rules: MappingRulesList


class RoleMapping(TypedDict, total=False):
    """A role mapping."""

    Type: RoleMappingType
    AmbiguousRoleResolution: Optional[AmbiguousRoleResolutionType]
    RulesConfiguration: Optional[RulesConfigurationType]


RoleMappingMap = Dict[IdentityProviderName, RoleMapping]
RolesMap = Dict[RoleType, ARNString]


class GetIdentityPoolRolesResponse(TypedDict, total=False):
    """Returned in response to a successful ``GetIdentityPoolRoles`` operation."""

    IdentityPoolId: Optional[IdentityPoolId]
    Roles: Optional[RolesMap]
    RoleMappings: Optional[RoleMappingMap]


TokenDuration = int
PrincipalTags = Dict[PrincipalTagID, PrincipalTagValue]


class GetOpenIdTokenForDeveloperIdentityInput(ServiceRequest):
    """Input to the ``GetOpenIdTokenForDeveloperIdentity`` action."""

    IdentityPoolId: IdentityPoolId
    IdentityId: Optional[IdentityId]
    Logins: LoginsMap
    PrincipalTags: Optional[PrincipalTags]
    TokenDuration: Optional[TokenDuration]


class GetOpenIdTokenForDeveloperIdentityResponse(TypedDict, total=False):
    """Returned in response to a successful
    ``GetOpenIdTokenForDeveloperIdentity`` request.
    """

    IdentityId: Optional[IdentityId]
    Token: Optional[OIDCToken]


class GetOpenIdTokenInput(ServiceRequest):
    """Input to the GetOpenIdToken action."""

    IdentityId: IdentityId
    Logins: Optional[LoginsMap]


class GetOpenIdTokenResponse(TypedDict, total=False):
    """Returned in response to a successful GetOpenIdToken request."""

    IdentityId: Optional[IdentityId]
    Token: Optional[OIDCToken]


class GetPrincipalTagAttributeMapInput(ServiceRequest):
    IdentityPoolId: IdentityPoolId
    IdentityProviderName: IdentityProviderName


class GetPrincipalTagAttributeMapResponse(TypedDict, total=False):
    IdentityPoolId: Optional[IdentityPoolId]
    IdentityProviderName: Optional[IdentityProviderName]
    UseDefaults: Optional[UseDefaults]
    PrincipalTags: Optional[PrincipalTags]


LoginsList = List[IdentityProviderName]


class IdentityDescription(TypedDict, total=False):
    """A description of the identity."""

    IdentityId: Optional[IdentityId]
    Logins: Optional[LoginsList]
    CreationDate: Optional[DateType]
    LastModifiedDate: Optional[DateType]


IdentitiesList = List[IdentityDescription]


class IdentityPool(ServiceRequest):
    """An object representing an Amazon Cognito identity pool."""

    IdentityPoolId: IdentityPoolId
    IdentityPoolName: IdentityPoolName
    AllowUnauthenticatedIdentities: IdentityPoolUnauthenticated
    AllowClassicFlow: Optional[ClassicFlow]
    SupportedLoginProviders: Optional[IdentityProviders]
    DeveloperProviderName: Optional[DeveloperProviderName]
    OpenIdConnectProviderARNs: Optional[OIDCProviderList]
    CognitoIdentityProviders: Optional[CognitoIdentityProviderList]
    SamlProviderARNs: Optional[SAMLProviderList]
    IdentityPoolTags: Optional[IdentityPoolTagsType]


class IdentityPoolShortDescription(TypedDict, total=False):
    """A description of the identity pool."""

    IdentityPoolId: Optional[IdentityPoolId]
    IdentityPoolName: Optional[IdentityPoolName]


IdentityPoolTagsListType = List[TagKeysType]
IdentityPoolsList = List[IdentityPoolShortDescription]


class ListIdentitiesInput(ServiceRequest):
    """Input to the ListIdentities action."""

    IdentityPoolId: IdentityPoolId
    MaxResults: QueryLimit
    NextToken: Optional[PaginationKey]
    HideDisabled: Optional[HideDisabled]


class ListIdentitiesResponse(TypedDict, total=False):
    """The response to a ListIdentities request."""

    IdentityPoolId: Optional[IdentityPoolId]
    Identities: Optional[IdentitiesList]
    NextToken: Optional[PaginationKey]


class ListIdentityPoolsInput(ServiceRequest):
    """Input to the ListIdentityPools action."""

    MaxResults: QueryLimit
    NextToken: Optional[PaginationKey]


class ListIdentityPoolsResponse(TypedDict, total=False):
    """The result of a successful ListIdentityPools action."""

    IdentityPools: Optional[IdentityPoolsList]
    NextToken: Optional[PaginationKey]


class ListTagsForResourceInput(ServiceRequest):
    ResourceArn: ARNString


class ListTagsForResourceResponse(TypedDict, total=False):
    Tags: Optional[IdentityPoolTagsType]


class LookupDeveloperIdentityInput(ServiceRequest):
    """Input to the ``LookupDeveloperIdentityInput`` action."""

    IdentityPoolId: IdentityPoolId
    IdentityId: Optional[IdentityId]
    DeveloperUserIdentifier: Optional[DeveloperUserIdentifier]
    MaxResults: Optional[QueryLimit]
    NextToken: Optional[PaginationKey]


class LookupDeveloperIdentityResponse(TypedDict, total=False):
    """Returned in response to a successful ``LookupDeveloperIdentity`` action."""

    IdentityId: Optional[IdentityId]
    DeveloperUserIdentifierList: Optional[DeveloperUserIdentifierList]
    NextToken: Optional[PaginationKey]


class MergeDeveloperIdentitiesInput(ServiceRequest):
    """Input to the ``MergeDeveloperIdentities`` action."""

    SourceUserIdentifier: DeveloperUserIdentifier
    DestinationUserIdentifier: DeveloperUserIdentifier
    DeveloperProviderName: DeveloperProviderName
    IdentityPoolId: IdentityPoolId


class MergeDeveloperIdentitiesResponse(TypedDict, total=False):
    """Returned in response to a successful ``MergeDeveloperIdentities``
    action.
    """

    IdentityId: Optional[IdentityId]


class SetIdentityPoolRolesInput(ServiceRequest):
    """Input to the ``SetIdentityPoolRoles`` action."""

    IdentityPoolId: IdentityPoolId
    Roles: RolesMap
    RoleMappings: Optional[RoleMappingMap]


class SetPrincipalTagAttributeMapInput(ServiceRequest):
    IdentityPoolId: IdentityPoolId
    IdentityProviderName: IdentityProviderName
    UseDefaults: Optional[UseDefaults]
    PrincipalTags: Optional[PrincipalTags]


class SetPrincipalTagAttributeMapResponse(TypedDict, total=False):
    IdentityPoolId: Optional[IdentityPoolId]
    IdentityProviderName: Optional[IdentityProviderName]
    UseDefaults: Optional[UseDefaults]
    PrincipalTags: Optional[PrincipalTags]


class TagResourceInput(ServiceRequest):
    ResourceArn: ARNString
    Tags: IdentityPoolTagsType


class TagResourceResponse(TypedDict, total=False):
    pass


class UnlinkDeveloperIdentityInput(ServiceRequest):
    """Input to the ``UnlinkDeveloperIdentity`` action."""

    IdentityId: IdentityId
    IdentityPoolId: IdentityPoolId
    DeveloperProviderName: DeveloperProviderName
    DeveloperUserIdentifier: DeveloperUserIdentifier


class UnlinkIdentityInput(ServiceRequest):
    """Input to the UnlinkIdentity action."""

    IdentityId: IdentityId
    Logins: LoginsMap
    LoginsToRemove: LoginsList


class UntagResourceInput(ServiceRequest):
    ResourceArn: ARNString
    TagKeys: IdentityPoolTagsListType


class UntagResourceResponse(TypedDict, total=False):
    pass


class CognitoIdentityApi:

    service = "cognito-identity"
    version = "2014-06-30"

    @handler("CreateIdentityPool")
    def create_identity_pool(
        self,
        context: RequestContext,
        identity_pool_name: IdentityPoolName,
        allow_unauthenticated_identities: IdentityPoolUnauthenticated,
        allow_classic_flow: ClassicFlow = None,
        supported_login_providers: IdentityProviders = None,
        developer_provider_name: DeveloperProviderName = None,
        open_id_connect_provider_arns: OIDCProviderList = None,
        cognito_identity_providers: CognitoIdentityProviderList = None,
        saml_provider_arns: SAMLProviderList = None,
        identity_pool_tags: IdentityPoolTagsType = None,
    ) -> IdentityPool:
        """Creates a new identity pool. The identity pool is a store of user
        identity information that is specific to your AWS account. The keys for
        ``SupportedLoginProviders`` are as follows:

        -  Facebook: ``graph.facebook.com``

        -  Google: ``accounts.google.com``

        -  Amazon: ``www.amazon.com``

        -  Twitter: ``api.twitter.com``

        -  Digits: ``www.digits.com``

        You must use AWS Developer credentials to call this API.

        :param identity_pool_name: A string that you provide.
        :param allow_unauthenticated_identities: TRUE if the identity pool supports unauthenticated logins.
        :param allow_classic_flow: Enables or disables the Basic (Classic) authentication flow.
        :param supported_login_providers: Optional key:value pairs mapping provider names to provider app IDs.
        :param developer_provider_name: The "domain" by which Cognito will refer to your users.
        :param open_id_connect_provider_arns: The Amazon Resource Names (ARN) of the OpenID Connect providers.
        :param cognito_identity_providers: An array of Amazon Cognito user pools and their client IDs.
        :param saml_provider_arns: An array of Amazon Resource Names (ARNs) of the SAML provider for your
        identity pool.
        :param identity_pool_tags: Tags to assign to the identity pool.
        :returns: IdentityPool
        :raises InvalidParameterException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises LimitExceededException:
        """
        raise NotImplementedError

    @handler("DeleteIdentities")
    def delete_identities(
        self, context: RequestContext, identity_ids_to_delete: IdentityIdList
    ) -> DeleteIdentitiesResponse:
        """Deletes identities from an identity pool. You can specify a list of 1-60
        identities that you want to delete.

        You must use AWS Developer credentials to call this API.

        :param identity_ids_to_delete: A list of 1-60 identities that you want to delete.
        :returns: DeleteIdentitiesResponse
        :raises InvalidParameterException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("DeleteIdentityPool")
    def delete_identity_pool(
        self, context: RequestContext, identity_pool_id: IdentityPoolId
    ) -> None:
        """Deletes an identity pool. Once a pool is deleted, users will not be able
        to authenticate with the pool.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("DescribeIdentity")
    def describe_identity(
        self, context: RequestContext, identity_id: IdentityId
    ) -> IdentityDescription:
        """Returns metadata related to the given identity, including when the
        identity was created and any associated linked logins.

        You must use AWS Developer credentials to call this API.

        :param identity_id: A unique identifier in the format REGION:GUID.
        :returns: IdentityDescription
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("DescribeIdentityPool")
    def describe_identity_pool(
        self, context: RequestContext, identity_pool_id: IdentityPoolId
    ) -> IdentityPool:
        """Gets details about a particular identity pool, including the pool name,
        ID description, creation date, and current number of users.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :returns: IdentityPool
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("GetCredentialsForIdentity")
    def get_credentials_for_identity(
        self,
        context: RequestContext,
        identity_id: IdentityId,
        logins: LoginsMap = None,
        custom_role_arn: ARNString = None,
    ) -> GetCredentialsForIdentityResponse:
        """Returns credentials for the provided identity ID. Any provided logins
        will be validated against supported login providers. If the token is for
        cognito-identity.amazonaws.com, it will be passed through to AWS
        Security Token Service with the appropriate role for the token.

        This is a public API. You do not need any credentials to call this API.

        :param identity_id: A unique identifier in the format REGION:GUID.
        :param logins: A set of optional name-value pairs that map provider names to provider
        tokens.
        :param custom_role_arn: The Amazon Resource Name (ARN) of the role to be assumed when multiple
        roles were received in the token from the identity provider.
        :returns: GetCredentialsForIdentityResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InvalidIdentityPoolConfigurationException:
        :raises InternalErrorException:
        :raises ExternalServiceException:
        """
        raise NotImplementedError

    @handler("GetId")
    def get_id(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        account_id: AccountId = None,
        logins: LoginsMap = None,
    ) -> GetIdResponse:
        """Generates (or retrieves) a Cognito ID. Supplying multiple logins will
        create an implicit linked account.

        This is a public API. You do not need any credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param account_id: A standard AWS account ID (9+ digits).
        :param logins: A set of optional name-value pairs that map provider names to provider
        tokens.
        :returns: GetIdResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises LimitExceededException:
        :raises ExternalServiceException:
        """
        raise NotImplementedError

    @handler("GetIdentityPoolRoles")
    def get_identity_pool_roles(
        self, context: RequestContext, identity_pool_id: IdentityPoolId
    ) -> GetIdentityPoolRolesResponse:
        """Gets the roles for an identity pool.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :returns: GetIdentityPoolRolesResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("GetOpenIdToken")
    def get_open_id_token(
        self, context: RequestContext, identity_id: IdentityId, logins: LoginsMap = None
    ) -> GetOpenIdTokenResponse:
        """Gets an OpenID token, using a known Cognito ID. This known Cognito ID is
        returned by GetId. You can optionally add additional logins for the
        identity. Supplying multiple logins creates an implicit link.

        The OpenID token is valid for 10 minutes.

        This is a public API. You do not need any credentials to call this API.

        :param identity_id: A unique identifier in the format REGION:GUID.
        :param logins: A set of optional name-value pairs that map provider names to provider
        tokens.
        :returns: GetOpenIdTokenResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises ExternalServiceException:
        """
        raise NotImplementedError

    @handler("GetOpenIdTokenForDeveloperIdentity")
    def get_open_id_token_for_developer_identity(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        logins: LoginsMap,
        identity_id: IdentityId = None,
        principal_tags: PrincipalTags = None,
        token_duration: TokenDuration = None,
    ) -> GetOpenIdTokenForDeveloperIdentityResponse:
        """Registers (or retrieves) a Cognito ``IdentityId`` and an OpenID Connect
        token for a user authenticated by your backend authentication process.
        Supplying multiple logins will create an implicit linked account. You
        can only specify one developer provider as part of the ``Logins`` map,
        which is linked to the identity pool. The developer provider is the
        "domain" by which Cognito will refer to your users.

        You can use ``GetOpenIdTokenForDeveloperIdentity`` to create a new
        identity and to link new logins (that is, user credentials issued by a
        public provider or developer provider) to an existing identity. When you
        want to create a new identity, the ``IdentityId`` should be null. When
        you want to associate a new login with an existing
        authenticated/unauthenticated identity, you can do so by providing the
        existing ``IdentityId``. This API will create the identity in the
        specified ``IdentityPoolId``.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param logins: A set of optional name-value pairs that map provider names to provider
        tokens.
        :param identity_id: A unique identifier in the format REGION:GUID.
        :param principal_tags: Use this operation to configure attribute mappings for custom providers.
        :param token_duration: The expiration time of the token, in seconds.
        :returns: GetOpenIdTokenForDeveloperIdentityResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises DeveloperUserAlreadyRegisteredException:
        """
        raise NotImplementedError

    @handler("GetPrincipalTagAttributeMap")
    def get_principal_tag_attribute_map(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        identity_provider_name: IdentityProviderName,
    ) -> GetPrincipalTagAttributeMapResponse:
        """Use ``GetPrincipalTagAttributeMap`` to list all mappings between
        ``PrincipalTags`` and user attributes.

        :param identity_pool_id: You can use this operation to get the ID of the Identity Pool you setup
        attribute mappings for.
        :param identity_provider_name: You can use this operation to get the provider name.
        :returns: GetPrincipalTagAttributeMapResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("ListIdentities")
    def list_identities(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        max_results: QueryLimit,
        next_token: PaginationKey = None,
        hide_disabled: HideDisabled = None,
    ) -> ListIdentitiesResponse:
        """Lists the identities in an identity pool.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param max_results: The maximum number of identities to return.
        :param next_token: A pagination token.
        :param hide_disabled: An optional boolean parameter that allows you to hide disabled
        identities.
        :returns: ListIdentitiesResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("ListIdentityPools")
    def list_identity_pools(
        self, context: RequestContext, max_results: QueryLimit, next_token: PaginationKey = None
    ) -> ListIdentityPoolsResponse:
        """Lists all of the Cognito identity pools registered for your account.

        You must use AWS Developer credentials to call this API.

        :param max_results: The maximum number of identities to return.
        :param next_token: A pagination token.
        :returns: ListIdentityPoolsResponse
        :raises InvalidParameterException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises ResourceNotFoundException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: ARNString
    ) -> ListTagsForResourceResponse:
        """Lists the tags that are assigned to an Amazon Cognito identity pool.

        A tag is a label that you can apply to identity pools to categorize and
        manage them in different ways, such as by purpose, owner, environment,
        or other criteria.

        You can use this action up to 10 times per second, per account.

        :param resource_arn: The Amazon Resource Name (ARN) of the identity pool that the tags are
        assigned to.
        :returns: ListTagsForResourceResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("LookupDeveloperIdentity")
    def lookup_developer_identity(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        identity_id: IdentityId = None,
        developer_user_identifier: DeveloperUserIdentifier = None,
        max_results: QueryLimit = None,
        next_token: PaginationKey = None,
    ) -> LookupDeveloperIdentityResponse:
        """Retrieves the ``IdentityID`` associated with a
        ``DeveloperUserIdentifier`` or the list of ``DeveloperUserIdentifier``
        values associated with an ``IdentityId`` for an existing identity.
        Either ``IdentityID`` or ``DeveloperUserIdentifier`` must not be null.
        If you supply only one of these values, the other value will be searched
        in the database and returned as a part of the response. If you supply
        both, ``DeveloperUserIdentifier`` will be matched against
        ``IdentityID``. If the values are verified against the database, the
        response returns both values and is the same as the request. Otherwise a
        ``ResourceConflictException`` is thrown.

        ``LookupDeveloperIdentity`` is intended for low-throughput control plane
        operations: for example, to enable customer service to locate an
        identity ID by username. If you are using it for higher-volume
        operations such as user authentication, your requests are likely to be
        throttled. GetOpenIdTokenForDeveloperIdentity is a better option for
        higher-volume operations for user authentication.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param identity_id: A unique identifier in the format REGION:GUID.
        :param developer_user_identifier: A unique ID used by your backend authentication process to identify a
        user.
        :param max_results: The maximum number of identities to return.
        :param next_token: A pagination token.
        :returns: LookupDeveloperIdentityResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("MergeDeveloperIdentities")
    def merge_developer_identities(
        self,
        context: RequestContext,
        source_user_identifier: DeveloperUserIdentifier,
        destination_user_identifier: DeveloperUserIdentifier,
        developer_provider_name: DeveloperProviderName,
        identity_pool_id: IdentityPoolId,
    ) -> MergeDeveloperIdentitiesResponse:
        """Merges two users having different ``IdentityId`` s, existing in the
        same identity pool, and identified by the same developer provider. You
        can use this action to request that discrete users be merged and
        identified as a single user in the Cognito environment. Cognito
        associates the given source user (``SourceUserIdentifier``) with the
        ``IdentityId`` of the ``DestinationUserIdentifier``. Only
        developer-authenticated users can be merged. If the users to be merged
        are associated with the same public provider, but as two different
        users, an exception will be thrown.

        The number of linked logins is limited to 20. So, the number of linked
        logins for the source user, ``SourceUserIdentifier``, and the
        destination user, ``DestinationUserIdentifier``, together should not be
        larger than 20. Otherwise, an exception will be thrown.

        You must use AWS Developer credentials to call this API.

        :param source_user_identifier: User identifier for the source user.
        :param destination_user_identifier: User identifier for the destination user.
        :param developer_provider_name: The "domain" by which Cognito will refer to your users.
        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :returns: MergeDeveloperIdentitiesResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("SetIdentityPoolRoles")
    def set_identity_pool_roles(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        roles: RolesMap,
        role_mappings: RoleMappingMap = None,
    ) -> None:
        """Sets the roles for an identity pool. These roles are used when making
        calls to GetCredentialsForIdentity action.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param roles: The map of roles associated with this pool.
        :param role_mappings: How users for a specific identity provider are to mapped to roles.
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("SetPrincipalTagAttributeMap")
    def set_principal_tag_attribute_map(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        identity_provider_name: IdentityProviderName,
        use_defaults: UseDefaults = None,
        principal_tags: PrincipalTags = None,
    ) -> SetPrincipalTagAttributeMapResponse:
        """You can use this operation to use default (username and clientID)
        attribute or custom attribute mappings.

        :param identity_pool_id: The ID of the Identity Pool you want to set attribute mappings for.
        :param identity_provider_name: The provider name you want to use for attribute mappings.
        :param use_defaults: You can use this operation to use default (username and clientID)
        attribute mappings.
        :param principal_tags: You can use this operation to add principal tags.
        :returns: SetPrincipalTagAttributeMapResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: ARNString, tags: IdentityPoolTagsType
    ) -> TagResourceResponse:
        """Assigns a set of tags to the specified Amazon Cognito identity pool. A
        tag is a label that you can use to categorize and manage identity pools
        in different ways, such as by purpose, owner, environment, or other
        criteria.

        Each tag consists of a key and value, both of which you define. A key is
        a general category for more specific values. For example, if you have
        two versions of an identity pool, one for testing and another for
        production, you might assign an ``Environment`` tag key to both identity
        pools. The value of this key might be ``Test`` for one identity pool and
        ``Production`` for the other.

        Tags are useful for cost tracking and access control. You can activate
        your tags so that they appear on the Billing and Cost Management
        console, where you can track the costs associated with your identity
        pools. In an IAM policy, you can constrain permissions for identity
        pools based on specific tags or tag values.

        You can use this action up to 5 times per second, per account. An
        identity pool can have as many as 50 tags.

        :param resource_arn: The Amazon Resource Name (ARN) of the identity pool.
        :param tags: The tags to assign to the identity pool.
        :returns: TagResourceResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("UnlinkDeveloperIdentity")
    def unlink_developer_identity(
        self,
        context: RequestContext,
        identity_id: IdentityId,
        identity_pool_id: IdentityPoolId,
        developer_provider_name: DeveloperProviderName,
        developer_user_identifier: DeveloperUserIdentifier,
    ) -> None:
        """Unlinks a ``DeveloperUserIdentifier`` from an existing identity.
        Unlinked developer users will be considered new identities next time
        they are seen. If, for a given Cognito identity, you remove all
        federated identities as well as the developer user identifier, the
        Cognito identity becomes inaccessible.

        You must use AWS Developer credentials to call this API.

        :param identity_id: A unique identifier in the format REGION:GUID.
        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param developer_provider_name: The "domain" by which Cognito will refer to your users.
        :param developer_user_identifier: A unique ID used by your backend authentication process to identify a
        user.
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("UnlinkIdentity")
    def unlink_identity(
        self,
        context: RequestContext,
        identity_id: IdentityId,
        logins: LoginsMap,
        logins_to_remove: LoginsList,
    ) -> None:
        """Unlinks a federated identity from an existing account. Unlinked logins
        will be considered new identities next time they are seen. Removing the
        last linked login will make this identity inaccessible.

        This is a public API. You do not need any credentials to call this API.

        :param identity_id: A unique identifier in the format REGION:GUID.
        :param logins: A set of optional name-value pairs that map provider names to provider
        tokens.
        :param logins_to_remove: Provider names to unlink from this identity.
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises ExternalServiceException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: ARNString, tag_keys: IdentityPoolTagsListType
    ) -> UntagResourceResponse:
        """Removes the specified tags from the specified Amazon Cognito identity
        pool. You can use this action up to 5 times per second, per account

        :param resource_arn: The Amazon Resource Name (ARN) of the identity pool.
        :param tag_keys: The keys of the tags to remove from the user pool.
        :returns: UntagResourceResponse
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        """
        raise NotImplementedError

    @handler("UpdateIdentityPool")
    def update_identity_pool(
        self,
        context: RequestContext,
        identity_pool_id: IdentityPoolId,
        identity_pool_name: IdentityPoolName,
        allow_unauthenticated_identities: IdentityPoolUnauthenticated,
        allow_classic_flow: ClassicFlow = None,
        supported_login_providers: IdentityProviders = None,
        developer_provider_name: DeveloperProviderName = None,
        open_id_connect_provider_arns: OIDCProviderList = None,
        cognito_identity_providers: CognitoIdentityProviderList = None,
        saml_provider_arns: SAMLProviderList = None,
        identity_pool_tags: IdentityPoolTagsType = None,
    ) -> IdentityPool:
        """Updates an identity pool.

        You must use AWS Developer credentials to call this API.

        :param identity_pool_id: An identity pool ID in the format REGION:GUID.
        :param identity_pool_name: A string that you provide.
        :param allow_unauthenticated_identities: TRUE if the identity pool supports unauthenticated logins.
        :param allow_classic_flow: Enables or disables the Basic (Classic) authentication flow.
        :param supported_login_providers: Optional key:value pairs mapping provider names to provider app IDs.
        :param developer_provider_name: The "domain" by which Cognito will refer to your users.
        :param open_id_connect_provider_arns: The ARNs of the OpenID Connect providers.
        :param cognito_identity_providers: A list representing an Amazon Cognito user pool and its client ID.
        :param saml_provider_arns: An array of Amazon Resource Names (ARNs) of the SAML provider for your
        identity pool.
        :param identity_pool_tags: The tags that are assigned to the identity pool.
        :returns: IdentityPool
        :raises InvalidParameterException:
        :raises ResourceNotFoundException:
        :raises NotAuthorizedException:
        :raises ResourceConflictException:
        :raises TooManyRequestsException:
        :raises InternalErrorException:
        :raises ConcurrentModificationException:
        :raises LimitExceededException:
        """
        raise NotImplementedError



from localstack.runtime import hooks
from localstack_ext.plugins import api_key_configured
@hooks.on_infra_start(should_load=api_key_configured)
def init_stackinfo_handler():from localstack.aws.handlers import run_custom_response_handlers as B;from localstack.services.internal import get_internal_apis as C;from .handler import StackInfoDataCollector as D;from .resource import StackInfoResource as E;A=D();B.append(A);C().add('/stackinfo',E(A))


_A='latest'
from typing import List
from localstack.constants import ARTIFACTS_REPO
from localstack.packages import DownloadInstaller,Package,PackageInstaller
class KafkaPackage(Package):
	def __init__(A):super().__init__(name='Kafka',default_version=_A)
	def get_versions(A):return[_A]
	def _get_installer(A,version):return KafkaPackageInstaller('kafka',version)
class KafkaPackageInstaller(DownloadInstaller):
	def _get_download_url(A):return f"{ARTIFACTS_REPO}/raw/master/kafka-server/build/libs/kafka-server-all.jar"
kafka_package=KafkaPackage()


_B='3.4.13'
_A='4.4.10'
import logging,os
from typing import List
from localstack.packages import DownloadInstaller,InstallTarget,Package,PackageInstaller
from localstack.packages.core import ArchiveDownloadAndExtractInstaller
from localstack.utils.archives import download_and_extract_with_retry
from localstack.utils.http import download
NEO4J_JAR_URL='https://dist.neo4j.org/neo4j-community-<ver>-unix.tar.gz'
TINKERPOP_ID_MANAGER_URL='https://github.com/localstack/localstack-artifacts/raw/f5a4d44ccc2a3f8ed1a584844f070fcf600b24f6/tinkerpop-id-manager/tinkerpop-id-manager.jar'
TINKERPOP_ID_MANAGER_FILE_NAME='tinkerpop-id-manager.jar'
LOG=logging.getLogger(__name__)
class Neo4JPackage(Package):
	def __init__(A):super().__init__('Neo4J',_A)
	def get_versions(A):return[_A]
	def _get_installer(A,version):return Neo4JPackageInstaller('neo4j',version)
class Neo4JPackageInstaller(ArchiveDownloadAndExtractInstaller):
	def _get_download_url(A):return NEO4J_JAR_URL.replace('<ver>',A.version)
	def _get_archive_subdir(A):return f"neo4j-community-{A.version}"
	def _get_install_marker_path(A,install_dir):return os.path.join(install_dir,f"neo4j-community-{A.version}",'bin','neo4j')
class TinkerpopPackage(Package):
	def __init__(A):super().__init__('Tinkerpop',_B)
	def get_versions(A):return[_B]
	def _get_installer(A,version):return TinkerpopPackageInstaller('tinkerpop',version)
class TinkerpopPackageInstaller(DownloadInstaller):
	def _get_download_url(A):return f"https://archive.apache.org/dist/tinkerpop/{A.version}/apache-tinkerpop-gremlin-server-{A.version}-bin.zip"
	def _install(A,target):
		C=target;B=A._get_install_dir(C);E=A._get_install_marker_path(B);D=os.path.join(A._get_install_dir(C),TINKERPOP_ID_MANAGER_FILE_NAME)
		if not os.path.exists(D):download(TINKERPOP_ID_MANAGER_URL,D)
		if not os.path.exists(E):LOG.debug('Downloading dependencies for Neptune Graph DB API (this may take some time) ...');F=os.path.join(B,'neptunedb.zip');download_and_extract_with_retry(A._get_download_url(),F,B)
	def _get_archive_subdir(A):return f"apache-tinkerpop-gremlin-server-{A.version}"
	def _get_install_marker_path(A,install_dir):return os.path.join(install_dir,f"apache-tinkerpop-gremlin-server-{A.version}",'bin','gremlin-server.sh')
neo4j_package=Neo4JPackage()
tinkerpop_package=TinkerpopPackage()


from localstack.packages import Package
from localstack_ext.packages.core import pro_package
@pro_package(name='mqtt')
def mosquitto_package():from localstack_ext.services.iot.packages import mosquitto_package as A;return A
@pro_package(name='iot-rule-engine')
def iot_rule_engine_package():from localstack_ext.services.iot.packages import iot_rule_engine_package as A;return A


from localstack.packages import Package
from localstack_ext.packages.core import pro_package
@pro_package(name='neo4j')
def neo4j_package():from localstack_ext.services.neptune.packages import neo4j_package as A;return A
@pro_package(name='tinkerpop')
def tinkerpop_package():from localstack_ext.services.neptune.packages import tinkerpop_package as A;return A


import io,logging,os,zipfile
from typing import Any,Callable,List
from localstack.utils.files import load_file,save_file
from localstack_ext.bootstrap.pods.service_state.service_state import ServiceState
from localstack_ext.bootstrap.pods.service_state.service_state_types import AssetNameType,AssetValueType,BackendState,BackendType,ServiceKey,ServiceNameType
from localstack_ext.constants import CLOUDPODS_METADATA_FILE
LOG=logging.getLogger(__name__)
class ServiceStateMarshaller:
	@staticmethod
	def marshall(state,marshall_function=None):
		F=marshall_function;E=state;B=io.BytesIO()
		with zipfile.ZipFile(B,'a')as G:
			for (C,J) in E.state.items():
				K=os.path.join(C.account_id,C.service,C.region)
				for (L,A) in J.backends.items():
					if not isinstance(A,bytes)and F:A=F(A)
					G.writestr(os.path.join('api_states',K,L),A)
			for (H,M) in E.assets.items():
				for (I,D) in M.items():
					if D is None:continue
					try:N=os.path.join(H,I);G.writestr(N,D)
					except Exception as O:LOG.exception('Failed to marshall %s for %s with value %s: %s',I,H,D,O)
		B.seek(0);return B.getvalue()
	@staticmethod
	def unmarshall(zip_content,raw_bytes=False,unmarshall_function=None):
		E=unmarshall_function;D=raw_bytes;A=zip_content
		if not A:return ServiceState()
		if not D and not E:LOG.debug('No unmarshal function provided')
		if isinstance(A,bytes):
			try:A=zipfile.ZipFile(io.BytesIO(A))
			except Exception as F:LOG.debug(f"Zip content not valid: {F}")
		C=ServiceState()
		def G(_filename):
			F=_filename;H=F.split(os.sep);I,G,J,K=H[-4:];B=A.read(F)
			if not D:
				try:B=E(B)
				except ModuleNotFoundError:LOG.warning('Unable to restore the state for %s',G);return
			L=BackendState(key=ServiceKey(I,J,G),backends={K:B});C.put_backend(L)
		def H(_filename):B=_filename;D=os.path.normpath(B).split(os.sep);E=D[0];F=os.path.join(*D[1:]);G=A.read(B);C.put_asset(E,F,G)
		for B in A.namelist():
			if not B.endswith('/'):
				if B.startswith('api_'):G(_filename=B)
				elif CLOUDPODS_METADATA_FILE in B:0
				else:H(_filename=B)
		return C
	@staticmethod
	def unmarshall_zip_archive(file_path):A=load_file(file_path,mode='rb');return ServiceStateMarshaller.unmarshall(A,raw_bytes=True)
	@staticmethod
	def marshall_zip_archive(file_path,service_state):
		A=service_state
		try:B=ServiceStateMarshaller.marshall(state=A)
		except Exception as C:LOG.exception('Failing to marshall service state: %s. Using original state',C);B=A
		save_file(file_path,B)


NIL_PTR = "NIL"

# well-defined directory names
ASSETS_ROOT_DIR = "assets"
# TODO: consider storing pods metadata in a non-hidden directory
DEFAULT_POD_DIR = "cloudpods"
OBJ_STORE_DIR = "objects"
VERSIONS_DIR = "versions"

# well-defined file names
HEAD_FILE = "HEAD"
VER_LOG_FILE = "VER_LOG"
MAX_VER_FILE = "MAX_VER"
KNOWN_VER_FILE = "KNOWN_VER"
VERSION_SPACE_FILES = [KNOWN_VER_FILE, MAX_VER_FILE, VER_LOG_FILE]
VERSION_SPACE_DIRS = [OBJ_STORE_DIR, VERSIONS_DIR]
VERSION_FILE = "version_{version_no}.yaml"
COMMIT_FILE = "metamodel_commit_{commit_no}.json"

# zip file names
META_ZIP = "version_{version_no}_meta"
STATE_ZIP = "version_{version_no}"
VERSION_SPACE_ARCHIVE = "version_space.zip"
COMPRESSION_FORMAT = "zip"

# misc. constants
VER_LOG_STRUCTURE = "{author};{ver_no};{rev_rid_no}"
